/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Viktor Pracht <viktor.pracht@open-xchange.com>
 * 
 */

  //////////////////
 //   LiveGrid   //
//////////////////

/**
 * The LiveGrid constructor.
 * @constructor
 * @param {Array} columns An array with column definitions.
 * Each column definition is an object with the following fields:
 * <ul><li>text is used as the column header,</li>
 * <li>i18n specifies whether the text is translated or used directly as
 * innerHTML (if text is an I18nString, i18n should be false),</li>
 * <li>index is used to extract the value of this column from an item object.
 * If not specified, the entire object is passed to {@link #set}.</li>
 * <li>width specifies the width of the column as a CSS length value,</li>
 * <li>style specifies an object with style attributes as elements,</li>
 * <li>sortable specifies whether sorting based on this column is supported,</li>
 * <li>id specifies the column ID,</li>
 * <lI>clear specifies a function which is called with an HTML DIV element as
 * parameter whenever the content of that DIV must be fetched from
 * the server. In the case of a static text in every row, {@link #makeClear}
 * can be used to create a function for this field.</li>
 * <li>set specifies a function which is called with an HTML DIV element and
 * the contents of that DIV whenever new contents must be displayed in the DIV.
 * In the case of pure text content, {@link #defaultSet} can be used
 * directly.</li></ul>
 * @param {Selection} selection The selection to use.
 * @param {String} module The name of a module for which to register Drag'n'Drop
 * functions.
 */
function LiveGrid(columns, selection, module) {
	this.disablehover=false;
	this.emptylivegridtext = _("This folder is empty");

	/**
	 * @private
	 */
	this.columns = columns;
	
	/**
	 * @private
	 */
	this.clickprocessed=false;

	/**
	 * The selection displayed in this LiveGrid.
	 */
	this.selection = selection || console.error("LiveGrid without Selection");
	
	/**
	 * @private
	 */
	this.module = module;

	/**
	 * Index of the anchor row.
	 * The anchor is used for multiple-selects with the Shift key.
	 * 0&nbsp;&lt;=&nbsp;anchor&nbsp;&lt;&nbsp;rowCount.
	 * @private
	 */
	this.anchor = 0;
	
	/**
	 * Index of the focus row.
	 * The focus is used for all selects.
	 * 0&nbsp;&lt;=&nbsp;focus&nbsp;&lt;&nbsp;rowCount.
	 * @private
	 */
	this.focus = 0;
	
	/**
	 * @private
	 * Sort order. Either "asc" or "desc".
	 */
	this.sort_order = "asc";
	
	/**
	 * @private
	 * ID of the column used for sorting. null if sorting is disabled.
	 */
	this.sort_id = null;
	
	for (var i = 0; i < columns.length; i++)
		if (!this.sort_id && columns[i].sortable) this.sort_id = columns[i].id;

	this.lineHeight = 1.6;
	
	/**
	 * @private
	 */
	this.from = 0;

	/**
	 * @private
	 */
	this.to = 0;
	
	/**
	 * @private
	 */
	this.rowcache = [];
	
	/**
	 * @private
	 * {Int: true}
	 */
	this.unfetched = {};

	var Self = this;

	/**
	 * @private
	 */
	this.fullRedraw = true;
	
	/**
	 * @private
	 */
	this.scroll_cb = function() { Self.scroll(); };
	
	/**
	 * @private
	 */
	this.key_cb = function(e) { return Self.key(e); };

	/**
	 * @private
	 */
	this.keydown_cb = function(e) {
		this.keydown = true;
		return Self.key(e);
	};
	
	/**
	 * @private
	 */
	this.keypress_cb = function(e) {
		if (this.keydown)
			this.keydown = false;
		else
			return Self.key(e);
	};
	
	/**
	 * @private
	 */
	this.mousedown_cb = function(e) { Self.mousedown(e); };
	
	/**
	 * @private
	 */
	this.ctxtmenu_cb = function(e) { Self.ctxtmenu(e); };
	
	/**
	 * @private
	 */
	this.click_cb = function(e) { Self.click(e); };
	
	this.language_cb = function(e) { Self.scroll(true); };
	/**
	 * @private
	 */
	this.dblclick_cb = function (e) {
	    // not top element?
        if (e.target !== Self.parent) {
            Self.dblclick(e);
        }
	};
	
	/**
	 * @private
	 */
	this.focus_cb = function() {
		focusedElement = Self.parent;
		Self.updateStyles();
	};
	
	/**
	 * @private
	 */
	this.blur_cb = function() {
		if (focusedElement == Self.parent)
			focusedElement = null;
		Self.updateStyles();
	};
	
	/**
	 * @private
	 */
	this.change_cb = function(from, to) {
		Self.rowCount = Self.storage.ids.length;
		from = Math.max(from, Self.from);
		to = Math.min(to, Self.to);
		for (var i = from; i < to; i++) Self.unfetched[i] = true;
		Self.doScroll();
		Self.updateStyles();
		Self.events.trigger("Changed", Self.storage);
	};
	
	/**
	 * @private
	 */
	this.selected_cb = function(count, e) {
		Self.updateStyles();
		Self.events.trigger("Selected", count, e);
	};
	
	/**
	 * Events triggered by this LiveGrid.
	 * <dl><dt>Selected</dt><dd>The selection has changed. Triggered only when
	 * this grid is enabled. Parameters:
	 * <ul><li>Number of selected elements.</li></ul></dd>
	 * <dt>Activated</dt><dd>One or more elements have been activated.
	 * Parameters:
	 * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
	 * <dt>Delete</dt><dd>One or more elements should be deleted.
	 * Parameters:
	 * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
	 * </dl>
	 */
	this.events = new Events();
	
	// update width (hack for portal refactoring; live grids grow endlessly)
	this.autoWidth = true;
}

function getRowIndex(e) {
	var n = e.target || e.srcElement;
	while (n.oxRowIndex == undefined && n.parentNode) n = n.parentNode;
	return n.oxRowIndex;
}

LiveGrid.prototype = {
	/**
	 * How many additional invisible rows to create to allow flicker-free
	 * scrolling at faster speeds.
	 */
	preview: 20,
	
	/**
	 * Granularity of row fetches.
	 */
	granularity: 1,
	
    /**
     * Recreates the LiveGrid, taking into account updated column definitions
     * etc.
     */
	recreate: function() {
        var storage = this.storage;
        var oldHeader = this.headerNode;
        this.disable();
        for (var i in this.rows) this.deleteRow(i);
        this.rowcache = [];
        this.from = 0;
        this.to = 0;
        this.ruWidth = 0;
        var header = this.getHeader();
        if (oldHeader && oldHeader.parentNode) {
            oldHeader.parentNode.replaceChild(header, oldHeader);
        }
        if (this.parent) {
            this.parent.removeChild(this.heightDiv);
            this.parent.removeChild(this.emptyMessage);
            this.getTable(this.parent);
        }
        if (this.hoverNode) {
            this.addHover(this.hoverNode, this.hoverCallback);
            if (this.hoverSizeNode) this.hover.setSize(this.hoverSizeNode);
        }
        if (storage) this.enable(storage);
    },
	
	/**
	 * Creates a DOM tree for the headers of the LiveGrid.
	 * The returned DOM tree must be inserted in its place in the document.
	 * @type Object
	 * @return A DOM tree which displays the headers of the LiveGrid.
	 * @see #getTable
	 */
	getHeader: function() {
		var Self = this;
		this.headers = new Array(this.columns.length);
		this.outer_headers = new Array(this.columns.length);
		this.arrows = new Array(this.columns.length);
		this.arrow_events = new Array(this.columns.length);
		for (var i in this.columns) this.arrow_events[i] = sortHandler(i);
		function sortHandler(i) {
			return function(e) {
				Self.setSort(Self.columns[i].id,
					(Self.sort_id == Self.columns[i].id &&
					 Self.sort_order == "asc") ? "desc" : "asc");
				stopEvent(e);
			}
		}
		this.ids = {};
		for (var i in this.columns) this.ids[this.columns[i].id] = i;
		var colgroup = this.getColGroup();
		colgroup.appendChild(this.ruCorner = newnode("col", {width: "0"}));
		var tbody = newnode("tbody",{verticalAlign : "top"});
		var table = this.headerNode = newnode("table", {tableLayout: "fixed"},
		                    {cellPadding: "0", cellSpacing: "0", className: "livegrid"},
		                    [colgroup, tbody]);
		var row = newnode("tr", {height: this.lineHeight + "em"},
		                  {className: "th"});
		for (var i = 0; i < this.columns.length; i++) {
			var outerCell, innerCell;
            var headerClassName = this.columns[i].name
                ? "header livegrid-column-" +
                      this.columns[i].name.join(" livegrid-column-")
                : "header";
			if (this.columns[i].sortable) {
				outerCell = this.outer_headers[i] =
					newnode("th", this.columns[i].style, {className: headerClassName}, [
						newnode("table", 0, {cellPadding: "0", cellSpacing: "0"}, [
							newnode("tbody", 0, 0, [
								newnode("tr", 0, 0, [
									innerCell = this.headers[i] = newnode("th", {whiteSpace: "nowrap"}),
									newnode("th", 0, 0, [
										this.arrows[i] = newnode("img", {verticalAlign: "middle"}, {src: getFullImgSrc("img/dummy.gif"), width: "16", height: "16"})
									])
								])
							])
						])
					]);
			} else
				innerCell = outerCell = this.headers[i] = this.outer_headers[i] =
					newnode("th", this.columns[i].style, {className: headerClassName});
			if (this.columns[i].text instanceof I18nString) {
                innerCell.appendChild(addTranslated(this.columns[i].text));
            } else if (this.columns[i].i18n) {
			    var ctx = this.columns[i].i18n;
                if (ctx === true) ctx = "";
				var node = new I18nNode(pgettext(ctx, this.columns[i].text));
				node.parent = innerCell;
				node.update = function() {
					this.parent.title = this.node.data = this.callback();
				};
				innerCell.appendChild(node.node);
				node.update();
			} else switch (typeof this.columns[i].text) {
                case "string":
				    innerCell.innerHTML = this.columns[i].text;
                    break;
                case "function":
                    innerCell.appendChild(addTranslated(this.columns[i].text));
                    break;
                default:
                    innerCell.appendChild(this.columns[i].text);
            }
			row.appendChild(outerCell);
		}
		row.appendChild(newnode("th"));
        var index = this.ids[this.sort_id];
		if (this.sort_id && this.arrows[index]) {
			this.arrows[index].src = 
				this.sort_order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
				this.sort_order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
				getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid.setSortClass(this.outer_headers[index], true);
        }
		tbody.appendChild(row);
		return table;
	},
	
	/**
	 * Adds the scrollable area of the LiveGrid to the specified parent node.
	 * The parent node should have an explicit size and "overflow: auto".
	 */
	getTable: function(parent) {
	    parent.onselectstart = function() { return false; };
		this.rows = {};
		var last = this.columns.length - 1;
		this.parent = parent;
		this.heightDiv = newnode("div", {height: "0em"}, {className: "livegrid"});
		parent.appendChild(this.heightDiv);
		parent.tabIndex = 0;
		parent.style.outline = "none";
		parent.style.overflowX = "hidden";
		parent.hideFocus = true;
		var emptyText = addTranslated(this.emptylivegridtext);
		this.emptyMessage = newnode("div",
			{display: "none", position: "absolute", textAlign: "center", 
			 width: "100%", marginTop: "5px"}, 0,
			[emptyText]);
		parent.appendChild(this.emptyMessage);
		var tr = newnode("tr");
		for (var i = 0; i < this.columns.length; i++)
			tr.appendChild(newnode("td", this.columns[i].style,
			                       {className: "cell"}));
		this.rowTemplate = newnode("table", 0,
			{cellPadding: "0", cellSpacing: "0", className: "tr"},
			[this.getColGroup(), newnode("tbody", 0, 0, [tr])]);
	},
	
	/**
	 * Adds a hover to the LiveGrid.
	 * This method should be called between getTable() and the first enable(). 
	 * @param {Object} hover A DOM node which serves as a hover. It should be
	 * attached directly to the body and have the CSS style
	 * "position: absolute".
     * @param {Function} callback A callback function which is called when
     * the hover is shown. As parameters, it receives the ID of
     * the entry over which the mouse is hovering and whether the hover was
     * triggered manually.
	 * @type {Object}
	 * @return A new Hover object. The enable and disable methods of the hover
	 * object will be called automatically by the LiveGrid.
	 */
	addHover: function(hover, callback) {
        this.hoverNode = hover;
	    this.hoverCallback = callback;
		this.hover = new Hover(this.heightDiv, hover);
		var Self = this;
        this.hover.setSize = function(node) {
            Self.hoverSizeNode = node;
            return Hover.prototype.setSize.call(this, node);
        }
		this.hover.onShow = function(node, manual) {
			var i = node.oxRowIndex;
			if (i !== undefined)
				callback(Self.storage.ids[i], manual);
			else
				return false;
		}
		return this.hover;
	},
	
	/**
	 * @private
	 */
	getColGroup: function() {
		var colgroup = newnode("colgroup");
		for (var i = 0; i < this.columns.length; i++) {
			var w = this.columns[i].width;
			colgroup.appendChild(newnode("col", w ? {width: w} : 0));
		}
		return colgroup;
	},
	
	/**
	 * Appends the contents either of the entire grid or of selected rows to
	 * the specified node.
	 * @param {Object} parent The node to append the data to.
	 * @param {Function} cb The callback to call after all data is appended.
	 * @param {Boolean} selected True if only selected rows should be appended.
	 * Otherwise, all rows are appended.
	 */
	print: function(parent, cb, selected) {
		var doc = parent.ownerDocument;
		var tr = doc.createElement("tr");
		for (var i = 0; i < this.columns.length; i++) {
			var th = doc.createElement("th");
			if (this.columns[i].i18n)
				th.appendChild(doc.createTextNode(_(this.columns[i].text)) || "\u00a0");
			else
				th.innerHTML = this.columns[i].text || "\u00a0";
			tr.appendChild(th);
		}
		var table = newnode("table", {width: "100%"}, {border: 1},
			[newnode("thead", 0, 0, [tr], doc)], doc);
		var trs = [];
		var Self = this;
		function set(n, obj) {
			trs[n] = doc.createElement("tr");
			for (var i = 0; i < Self.columns.length; i++) {
				var td = doc.createElement("td");
				trs[n].appendChild(td);
				var col = Self.columns[i];
				col.set(td, col.index ? obj[col.index] : obj, obj);
			}
		}
		function final_() {
			var tbody = newnode("tbody", 0, 0, 0, doc);
			for (var i = 0; i < trs.length; i++) 
				if (trs[i]) tbody.appendChild(trs[i]);
			table.appendChild(tbody);
			parent.appendChild(table);
			if(trs.length==0) {
				//return; 
			} 
			cb();
		}
		this.storage.newIterate(
			selected ? this.selection.getSelected() : this.storage.ids,
			emptyFunction, set, final_);
	},
	
	/**
	 * Displays the specified sort order as an arrow in the headers.
	 * @param {String} id The column ID used for sorting, or null for no
	 * sorting.
	 * @param {String} order The sort order. One of "asc", "desc" or null for no
	 * sorting.
	 */
	setSort: function(id, order) {
		if (this.sort_id && id != this.sort_id) {
            var index = this.ids[this.sort_id];
			this.arrows[index].src =
                getFullImgSrc("img/dummy.gif");
            LiveGrid.setSortClass(this.outer_headers[index], false);
        }
        var index = this.ids[id];
        if ((index || index === 0) && this.arrows[index] !== undefined) {
    		this.arrows[index].src = 
    			order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
    			order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
    			getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid.setSortClass(this.outer_headers[index], true);
        }
		this.sort_id = id;
		this.sort_order = order;
		if (this.storage) this.sort(id, order);
	},
	
	/**
	 * Callback function which changes the displayed data.
	 * Should be overridden in each instance.
	 * @param {String} id The column ID used for sorting, or null for no
	 * sorting.
	 * @param {String} order The sort order. Either "asc" or "desc".
	 */
	sort: emptyFunction,

	getSelectedIDs: function() {
		return this.selection.getSelected();
	},
	
	/**
	 * Deletes items with specified object IDs from the storage and from
	 * the selection. If the entire selection was deleted, the first unselected
	 * item at or after the focus is selected.
	 */
	deleteIDs: function (ids) {
	    if (this.storage) {
    		var sids = {};
    		for (var i = 0; i < ids.length; i++)
    			sids[this.storage.serialize(ids[i])] = true;
    		while (this.storage.getSID(this.focus) in sids) this.focus++;
    		var newID = this.storage.ids[this.focus];
    		this.storage.removeIDs(ids);
    		this.selection.deselectSIDs(sids);
    		this.focus = newID ? this.storage.getIndex(newID) : 0;
    		if (!this.selection.count)
    			this.selection.click(this.focus);
	    }
	},
	
	updateSelected: function(callback) {
		if (this.storage) {
			this.storage.localUpdate(this.selection.getSelected(), callback);
		}
	},

	/**
	 * Initializes the LiveGrid.
	 * The DOM tree must have been created by {@link #getHeader} and
	 * {@link #getTable}.
	 * @see #getHTML
	 */
	enable: function(storage) {
		if (this.storage) {
			this.storage.events.unregister("Changed", this.change_cb);
		} else {
			for (var i in this.arrows)
				addDOMEvent(this.outer_headers[i], "click", this.arrow_events[i]);
			addDOMEvent(this.parent, "scroll", this.scroll_cb);
			addDOMEvent(this.parent, "keydown", this.keydown_cb);
			addDOMEvent(this.parent, "keypress", this.keypress_cb);
			addDOMEvent(this.parent, "mousedown", this.mousedown_cb);
			addDOMEvent(this.parent, "click", this.click_cb);
			// bind double click via jQuery (due to bug #14560)
            jQuery(this.parent).bind("dblclick", this.dblclick_cb);
			addDOMEvent(this.parent, "focus", this.focus_cb);
			addDOMEvent(this.parent, "blur", this.blur_cb);
            if (this.contextmenu)
                addDOMEvent(this.parent, "contextmenu", this.ctxtmenu_cb);
			if (this.hover && !this.disablehover) this.hover.enable();
			resizeEvents.register("Resized", this.scroll_cb);
			register("LanguageChanged",this.language_cb);
			if (this.module && (this.module != "links")) {
				var Self = this;
				var disabled;
				var validfield;
				switch(this.module) {
					case "calendar": disabled=calendardefaultdisabled;	validfield=5; break;
					case "tasks":	disabled=taskdefaultdisabled; validfield=3; break;
					case "contacts": disabled=contactdefaultdisabled; validfield=2; break;
					case "infostore": disabled=infostoredefaultdisabled; validfield=3; break;
					case "mail": disabled=maildefaultdisabled; validfield=5; break;
				}
				this.dragEvent = registerSource(this.heightDiv, this.module, function() {
					var myelements=clone(Self.selection.getSelected());
					if(myelements.length==1) {
						Self.storage.newIterate(myelements,emptyFunction,function (index,id) {
							myelements[0].livegridfield=id[validfield];
						})
					}
					objMoveCopy.begin(Self.module,myelements);
					return objMoveCopy;
					
				},null,null,disabled,defaultdisabledremove);
			}
		}
		storage.events.register("Changed", this.change_cb);
		this.storage = storage;
		this.rowCount = storage.ids.length;
		this.selection.setStorage(storage);
		this.selection.events.register("Selected", this.selected_cb);
		this.scroll(true);
	},
	
	disable: function() {
		if (!this.storage) return;
		this.selection.events.unregister("Selected", this.selected_cb);
		this.storage.events.unregister("Changed", this.change_cb);
		this.storage = null;
		this.selection.setStorage(null);
		for (var i in this.arrows)
			removeDOMEvent(this.outer_headers[i], "click", this.arrow_events[i]);
		removeDOMEvent(this.parent, "scroll", this.scroll_cb);
		removeDOMEvent(this.parent, "keydown", this.keydown_cb);
		removeDOMEvent(this.parent, "keypress", this.keypress_cb);
		removeDOMEvent(this.parent, "mousedown", this.mousedown_cb);
		removeDOMEvent(this.parent, "click", this.click_cb);
		jQuery(this.parent).unbind("dblclick", this.dblclick_cb);
		removeDOMEvent(this.parent, "focus", this.focus_cb);
		removeDOMEvent(this.parent, "blur", this.blur_cb);
        if (this.contextmenu)
            removeDOMEvent(this.parent, "contextmenu", this.ctxtmenu_cb);
		if (this.hover) this.hover.disable();
		resizeEvents.unregister("Resized", this.scroll_cb);
		unregister("LanguageChanged",this.language_cb);
		if (this.module) {
			unregisterSource(this.parent, this.dragEvent);
		}
	},
	
	/**
	 * @private
	 */
	createRow: function() {
		var last = this.columns.length - 1;
		var row = {tr: this.rowTemplate.cloneNode(true),
		           cells: new Array(this.columns.length)};
		for (var i = 0, cell = row.tr.lastChild.firstChild.firstChild; cell;
		     i++, cell = cell.nextSibling)
		{
			row.cells[i] = cell;
			this.columns[i].clear(cell);
		}
		this.heightDiv.appendChild(row.tr);
		if (!this.rowHeight) this.rowHeight = row.tr.offsetHeight;
		return row;
	},

	/**
	 * To be removed.
	 * @private
	 */
	updateStyles: function() {
		for (var i in this.rows) {
			var newclass = this.selection.get(i) ? "tr selected" : i%2 ? "tr_h " : "tr ";
			if (focusedElement == this.parent && i == this.focus)
				newclass += "focused";
			var tr = this.rows[i].tr;
			if (tr.className != newclass) tr.className = newclass;
		}
	},

	/**
	 * @private
	 */
	deleteRow: function(index) {
		var row = this.rows[index];
		row.tr.style.display = "none";
		this.rowcache.push(row);
		delete this.rows[index];
		delete this.unfetched[index];
	},

	/**
	 * @private
	 */
	insertRow: function(index) {
		var row = this.rows[index] =
			this.rowcache.length ? this.rowcache.pop() : this.createRow();
		row.tr.className = this.selection.get(index) ? "tr selected" : index%2 ? "tr_h" : "tr";
		row.tr.style.display = "";
		row.tr.style.top = (index * this.rowHeight) + "px";
		row.tr.oxRowIndex = index;
		this.unfetched[index] = true;
	},

	/**
	 * @private
	 */
	scroll: function(fullRedraw) {
		if (fullRedraw) this.fullRedraw = true;
		if (this.scrollTimeout) clearTimeout(this.scrollTimeout);
		var Self = this;
		this.scrollTimeout = setTimeout(function() {
			Self.scrollTimeout = null;
			Self.doScroll();
		}, 0);
	},
	
	/**
	 * @private
	 */
	calculateRowHeight: function() {
		var row = this.createRow();
		row.tr.style.display = "none";
		this.rowcache.push();
	},

	/**
	 * @private
	 */
	doScroll: function() {
		var newruWidth = this.parent.offsetWidth - this.parent.clientWidth;
		if (newruWidth > 0 && newruWidth != this.ruWidth) {
			this.ruCorner.style.width = newruWidth + "px";
			this.ruWidth = newruWidth;
		}
		if (!this.rowHeight) this.calculateRowHeight();
		this.height = Math.floor(this.parent.clientHeight / this.rowHeight);
		var len = this.storage ? this.storage.ids.length : 0;
		var newHeight = (len * this.rowHeight) + "px"
		var hDivStyle = this.heightDiv.style;
		if (newHeight != hDivStyle.height) hDivStyle.height = newHeight;
		var Self = this;
		// update width?
		if (this.autoWidth) {
		    if (this.heightDiv.offsetWidth != this.parent.clientWidth) {
		        hDivStyle.width = Self.parent.clientWidth + "px";
		    }
		}
		var scrolltop = this.parent.scrollTop;
		var from2 = Math.max(0, Math.floor(                           scrolltop  / this.rowHeight / this.granularity) * this.granularity - this.preview);
		var to2 = Math.min(len, Math.ceil((this.parent.clientHeight + scrolltop) / this.rowHeight / this.granularity) * this.granularity + this.preview);
		var Self = this;
		if (this.fullRedraw || from2 != this.from || to2 != this.to) {
			if (this.fullRedraw || from2 > this.to || this.from > to2) {
				for (var i = this.from; i < this.to; i++) this.deleteRow(i);
				for (var i = from2; i < to2; i++) this.insertRow(i);
			} else {
				for (var i = this.from; i < from2; i++) this.deleteRow(i);
				for (var i = to2; i < this.to; i++) this.deleteRow(i);
				for (var i = from2; i < this.from; i++) this.insertRow(i);
				for (var i = this.to; i < to2; i++) this.insertRow(i);
			}
			this.from = from2;
			this.to = to2;
		}
		this.fullRedraw = false;
        this.emptyMessage.style.display = (!this.storage || len) ? "none"
                                                                : "block";
		var ids = [];
		for (var i in this.unfetched) ids.push(this.storage.ids[i]);
		if (ids.length) {
			if (this.iterateRequest) this.storage.cancel(this.iterateRequest);
			this.iterateRequest = this.storage.newIterate(ids, function(n) {
				if (!(n in Self.unfetched)) return;
				var cellrow = Self.rows[n].cells;
				for (var i in cellrow) Self.columns[i].clear(cellrow[i]);
			}, function(n, obj) {
				if (!(n in Self.unfetched)) return;
				delete Self.unfetched[n];
                var cellrow = Self.rows[n].cells;
				for (var i in cellrow) {
					var col = Self.columns[i];
					col.set(cellrow[i], col.index ? obj[col.index] : obj, obj);
				}
			}, function() { Self.iterateRequest = null; }, 20);
		}
	},
	
	/**
	 * @private
	 */
	scrollTo: function(y) {
		var parent = this.parent;
		setTimeout(function() { parent.scrollTop = y; }, 0);
	},
	
	/**
	 * Scrolls if necessary to display the currently focused row.
	 */
	showFocus: function() {
		var scrolltop = this.parent.scrollTop;
		if (!this.rowHeight) this.calculateRowHeight();
		if (this.focus < scrolltop / this.rowHeight)
			this.scrollTo(this.focus * this.rowHeight);
		else {
			var height = this.parent.clientHeight;
			if (this.focus + 1 > (scrolltop + height) / this.rowHeight)
				this.scrollTo((this.focus + 1) * this.rowHeight - height);
		}
	},

	/**
	 * @private
	 */
	key: function(e) {
		if (!this.storage) return;
		switch (e.keyCode || e.which) {
		case 13: // enter
			this.events.trigger("Activated", this.selection.getSelected());
			break;
		case 32: // spacebar
			this.selection.click(this.focus, e);
			this.showFocus();
			break;
		case 46: // delete
		case 108: // numpad delete
			this.events.trigger("Delete", this.selection.getSelected());
			break;
		case 38: // arrow up
			if (this.focus > 0) this.focus--;
			if (!(Mac ? e.metaKey : e.ctrlKey))
				this.selection.click(this.focus, e);
			this.showFocus();
			break;
		case 40: // arrow down
			if (this.focus < this.rowCount - 1) this.focus++;
			if (!(Mac ? e.metaKey : e.ctrlKey)) {
				this.selection.click(this.focus, e);
			}
			this.showFocus();
			break;
		case 33: // page up
			this.focus -= this.height;
			if (this.focus < 0) this.focus = 0;
			if (!(Mac ? e.metaKey : e.ctrlKey))
				this.selection.click(this.focus, e);
			this.showFocus();
			break;
		case 34: // page down
			this.focus += this.height;
			if (this.focus >= this.rowCount) this.focus = this.rowCount - 1;
			if (!(Mac ? e.metaKey : e.ctrlKey))
				this.selection.click(this.focus, e);
			this.showFocus();
			break;
		case 35: // end
			this.focus = this.rowCount - 1;
			if (!(Mac ? e.metaKey : e.ctrlKey))
				this.selection.click(this.focus, e);
			this.showFocus();
			break;
		case 36: // home
			this.focus = 0;
			if (!(Mac ? e.metaKey : e.ctrlKey))
				this.selection.click(this.focus, e);
			this.showFocus();
			break;
		case 65: // A
		case 97: // a
			if (e.ctrlKey)
				this.selection.select(0, this.storage.ids.length);
			break;
		default:
			return;
		}
		this.updateStyles();
		stopEvent(e);
		return false;
	},
	
    /**
     * @private
     */
    mousedown: function(e) {
        this.clickprocessed = false;
        var focus = getRowIndex(e);
        if (this.storage.ids.length < this.height) {
            if (focus === undefined) focus = -1; else this.focus = focus;
            if(!this.selection.get(focus)) {
                this.clickprocessed=true;
                this.selection.click(focus, e);
            }
        } else if (focus !== undefined) {
            this.focus = focus;
            if(!this.selection.get(focus)) {
                this.clickprocessed=true;
                this.selection.click(focus, e);
            }
        }
        setFocus(this.parent);
    },

	/**
	 * @private
	 */
	ctxtmenu: function(e) {
		stopEvent(e);
        this.contextmenu.display(e.clientX, e.clientY, this.selection);
	},
	
	/**
	 * @private
	 */
	click: function(e) {
		if (this.clickprocessed) return;
		var focus = getRowIndex(e);
		if (this.storage.ids.length < this.height) {
			if (focus === undefined) focus = -1; else this.focus = focus;
			this.selection.click(focus, e);
		} else if (focus !== undefined) {
			this.focus = focus;
			this.selection.click(focus, e);
		}
		setFocus(this.parent);
	},
	
	/**
	 * @private
	 */
	dblclick: function(e) {
		if (!this.storage) return;
		this.focus = getRowIndex(e) || 0;
		this.events.trigger("Activated", this.selection.getSelected());
	}
};

LiveGrid.setSortClass = classNameSetter("livegrid-th-sorted");


/**
 * Default set function for use in the second parameter of the constructor.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified content value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} text The text to set in the DIV element.
 */
LiveGrid.defaultSet = function(div, text) {
	text = text || "\u00a0";
	if (div.firstChild)
		div.firstChild.data = text;
	else
		div.appendChild(document.createTextNode(text));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid.dateSet = function(div, date) {
	var dateString = "\u00a0";
	if (null != date) dateString = formatDate(date, "date");
	if (div.firstChild)
		div.firstChild.data = dateString;
	else
		div.appendChild(document.createTextNode(dateString));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid.dateSetTime = function(div, date) {
    var dateString = "\u00a0";
    if (null != date) dateString = formatDate(date, "datetime");
	if (div.firstChild)
		div.firstChild.data = dateString
	else
		div.appendChild(document.createTextNode(dateString));
};

/**
 * Creates a clear function for use in the second parameter of the constructor.
 * This function creates a function which sets all DIV elements of the LiveGrid
 * to the text which is specified as parameter to this function.
 * @param {String} text The text to set in DIV elements.
 * @type Function
 * @return A function which takes a HTML DIV element as parameter, adds a text
 * node to it, and sets the text to a constant value.
 */
LiveGrid.makeClear = function(text) {
	return function(div) {
		if (div.firstChild)
			div.firstChild.data = text;
		else
			div.appendChild(document.createTextNode(text));
	};
};

/**
 * Sort function for use with purely local storages.
 */
LiveGrid.localSort = function(id, order) {
    var a = [];
    var Self = this;
    this.storage.newIterate(this.storage.ids, emptyFunction,
        function(div, data) { a.push(data); }, function() {
            Self.storage.remove(0, Self.storage.ids.length);
            a.sort(order == "asc" ? function(a, b) {
                return a[id] > b[id] ? 1 : a[id] < b[id] ? -1 : 0;
            } : function(b, a) {
                return a[id] > b[id] ? 1 : a[id] < b[id] ? -1 : 0;
            });
            Self.storage.append(a);
        });
};















function LiveGrid2(columns, selection, module) {
    this.disablehover=false;
    this.emptylivegridtext = _("This folder is empty");

    /**
     * @private
     */
    this.columns = columns;
    
    /**
     * @private
     */
    this.clickprocessed=false;

    /**
     * The selection displayed in this LiveGrid2.
     */
    this.selection = selection || console.error("LiveGrid2 without Selection");
    
    /**
     * @private
     */
    this.module = module;
    this.dndModule = module;
    /**
     * Index of the anchor row.
     * The anchor is used for multiple-selects with the Shift key.
     * 0&nbsp;&lt;=&nbsp;anchor&nbsp;&lt;&nbsp;rowCount.
     * @private
     */
    this.anchor = 0;
    
    /**
     * Index of the focus row.
     * The focus is used for all selects.
     * 0&nbsp;&lt;=&nbsp;focus&nbsp;&lt;&nbsp;rowCount.
     * @private
     */
    this.focus = 0;
    
    /**
     * @private
     * Sort order. Either "asc" or "desc".
     */
    this.sort_order = "asc";
    
    /**
     * @private
     * ID of the column used for sorting. null if sorting is disabled.
     */
    this.sort_id = null;
    
    for (var i = 0; i < columns.length; i++)
        if (!this.sort_id && columns[i].sortable) this.sort_id = columns[i].sort;

    this.lineHeight = 1.6;
    this.rowHeight = 0;
    this.rowMargin = 0;
    
    /**
     * @private
     */
    this.from = 0;

    /**
     * @private
     */
    this.to = 0;
    
    /**
     * @private
     */
    this.rowcache = [];
    
    /**
     * @private
     * {Int: true}
     */
    this.unfetched = {};

    var Self = this;

    /**
     * @private
     */
    this.fullRedraw = true;
    
    /**
     * @private
     */
    this.scroll_cb = function() { Self.scroll(); };
    
    /**
     * @private
     */
    this.key_cb = function(e) { return Self.key(e); };

    /**
     * @private
     */
    this.keydown_cb = function(e) {
        this.keydown = true;
        return Self.key(e);
    };
    
    /**
     * @private
     */
    this.keypress_cb = function(e) {
        if (this.keydown)
            this.keydown = false;
        else
            return Self.key(e);
    };
    
    /**
     * @private
     */
    this.mousedown_cb = function(e) { Self.mousedown(e); };
    
    /**
     * @private
     */
    this.ctxtmenu_cb = function(e) { Self.ctxtmenu(e); };
    
    /**
     * @private
     */
    this.click_cb = function(e) { Self.click(e); };
    
    /**
     * @private
     */
    this.dblclick_cb = function (e) {
    	Self.dblclick(e);
    };
    
    /**
     * @private
     */
    this.focus_cb = function() {
        focusedElement = Self.parent;
        Self.updateStyles();
    };
    
    this.language_cb = function(e) { Self.scroll(true); }
    
    /**
     * @private
     */
    this.blur_cb = function() {
        if (focusedElement == Self.parent)
            focusedElement = null;
        Self.updateStyles();
    };
    
    /**
     * @private
     */
    this.change_cb = function(from, to) {
        Self.rowCount = Self.collection.objects.length;
        from = Math.max(from, Self.from);
        to = Math.min(to, Self.to);
        for (var i = from; i < to; i++) Self.unfetched[i] = true;
        Self.doScroll();
        Self.updateStyles();
    };
    
    /**
     * @private
     */
    this.selected_cb = function(count,e ) {
    	Self.updateStyles();
        Self.events.trigger("Selected", count, e);
    };
    
    /**
     * Events triggered by this LiveGrid2.
     * <dl><dt>Selected</dt><dd>The selection has changed. Triggered only when
     * this grid is enabled. Parameters:
     * <ul><li>Number of selected elements.</li></ul></dd>
     * <dt>Activated</dt><dd>One or more elements have been activated.
     * Parameters:
     * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
     * <dt>Delete</dt><dd>One or more elements should be deleted.
     * Parameters:
     * <ul><li>Array with object IDs of activated objects.</li></ul></dd>
     * </dl>
     */
    this.events = new Events();
}

LiveGrid2.prototype = {
    constructor : "LiveGrid2",
    /**
     * How many additional invisible rows to create to allow flicker-free
     * scrolling at faster speeds.
     */
    preview: 15,
    
    /**
     * Granularity of row fetches.
     */
    granularity: 15,
    
    /**
     * Recreates the LiveGrid, taking into account updated column definitions
     * etc.
     */
    recreate: function() {
        var enabled = Boolean(this.collection);
        var oldHeader = this.headerNode;
        this.disable();
        for (var i in this.rows) this.deleteRow(i);
        this.rowcache = [];
        this.from = 0;
        this.to = 0;
        this.ruWidth = 0;
        var header = this.getHeader();
        if (oldHeader && oldHeader.parentNode) {
            oldHeader.parentNode.replaceChild(header, oldHeader);
        }
        if (this.parent) {
            this.parent.removeChild(this.heightDiv);
            this.parent.removeChild(this.emptyMessage);
            this.getTable(this.parent);
        }
        if (this.hoverNode) {
            this.addHover(this.hoverNode, this.hoverCallback);
            if (this.hoverSizeNode) this.hover.setSize(this.hoverSizeNode);
        }
        if (enabled) this.enable(this.collectiondata.criteria,
                                 this.collectiondata.search);
    },
    
    /**
     * Creates a DOM tree for the headers of the LiveGrid2.
     * The returned DOM tree must be inserted in its place in the document.
     * @type Object
     * @return A DOM tree which displays the headers of the LiveGrid2.
     * @see #getTable
     */
    getHeader: function() {
        var Self = this;
        this.headers = new Array(this.columns.length);
        this.outer_headers = new Array(this.columns.length);
        this.arrows = new Array(this.columns.length);
        this.arrow_events = new Array(this.columns.length);
        for (var i in this.columns) this.arrow_events[i] = sortHandler(i);
        function sortHandler(i) {
            return function(e) {
                Self.setSort(Self.columns[i].sort,
                    (Self.sort_id == Self.columns[i].sort &&
                     Self.sort_order == "asc") ? "desc" : "asc");
                stopEvent(e);
            }
        }
        this.ids = {};
        for (var i in this.columns) {
			if(this.columns[i].sort) {
		        this.ids[this.columns[i].sort] = i;		
			}
		}
        this.listColumns = function() {
        	var colSet = {};
            for(var i = 0; i < this.columns.length; i++) {
                var name = this.columns[i].name;
                for(var j = 0; j < name.length; j++) colSet[name[j]] = true;  
            }
            var plainColumns = [], eavColumns = [];
            for (var i in colSet) {
                (i.indexOf("/") < 0 ? plainColumns : eavColumns).push(i);
            }
            eavColumns.sort();
            return plainColumns.concat(eavColumns);
        };
        
        var colgroup = this.getColGroup();
        colgroup.appendChild(this.ruCorner = newnode("col", {width: "0"}));
        var tbody = newnode("tbody",{verticalAlign : "top"});
        var table = this.headerNode = newnode("table", {tableLayout: "fixed"},
                            {cellPadding: "0", cellSpacing: "0", className: "livegrid"},
                            [colgroup, tbody]);
        var row = newnode("tr", {height: this.lineHeight + "em"},
                          {className: "th"});
        for (var i = 0; i < this.columns.length; i++) {
            var outerCell, innerCell;
            var headerClassName = this.columns[i].name
                ? "header livegrid-column-" +
                      this.columns[i].name.join(" livegrid-column-")
                : "header";
            if (this.columns[i].sortable) {
                outerCell = this.outer_headers[i] =
                    newnode("th", this.columns[i].style, {className: headerClassName}, [
                        newnode("table", 0, {cellPadding: "0", cellSpacing: "0"}, [
                            newnode("tbody", 0, 0, [
                                newnode("tr", 0, 0, [
                                    innerCell = this.headers[i] = newnode("th", {whiteSpace: "nowrap"}),
                                    newnode("th", 0, 0, [
                                        this.arrows[i] = newnode("img", {verticalAlign: "middle"}, {src: getFullImgSrc("img/dummy.gif"), width: "16", height: "16"})
                                    ])
                                ])
                            ])
                        ])
                    ]);
            } else
                innerCell = outerCell = this.headers[i] = this.outer_headers[i] =
                    newnode("th", this.columns[i].style, {className: headerClassName});
            if (this.columns[i].text instanceof I18nString) {
                innerCell.appendChild(addTranslated(this.columns[i].text));
            } else if (this.columns[i].i18n) {
                var ctx = this.columns[i].i18n;
                if (ctx === true) ctx = ""; 
                var node = new I18nNode(pgettext(ctx, this.columns[i].text));
                node.parent = innerCell;
                node.update = function() {
                    this.parent.title = this.node.data = this.callback();
                };
                innerCell.appendChild(node.node);
                node.update();
            } else switch (typeof this.columns[i].text) {
                case "string":
                    innerCell.innerHTML = this.columns[i].text;
                    break;
                case "function":
                    innerCell.appendChild(addTranslated(this.columns[i].text));
                    break;
                default:
                    innerCell.appendChild(this.columns[i].text);
            }
            row.appendChild(outerCell);
        }
        row.appendChild(newnode("th"));
        var index = this.ids[this.sort_id];
        if (this.sort_id && index && this.arrows[index]) {
            this.arrows[index].src = 
                this.sort_order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
                this.sort_order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
                getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid2.setSortClass(this.outer_headers[index], true);
         }
        tbody.appendChild(row);
        return table;
    },
    
    /**
     * Adds the scrollable area of the LiveGrid2 to the specified parent node.
     * The parent node should have an explicit size and "overflow: auto".
     */
    getTable: function(parent) {
        parent.onselectstart = function() { return false; };
        this.rows = {};
        var last = this.columns.length - 1;
        this.parent = parent;
        this.heightDiv = newnode("div", {height: "0em"}, {className: "livegrid"});
        parent.appendChild(this.heightDiv);
        parent.tabIndex = 0;
        parent.style.outline = "none";
        parent.style.overflowX = "hidden";
        parent.hideFocus = true;
        var emptyText = addTranslated(this.emptylivegridtext);
        this.emptyMessage = newnode("div",
            {display: "none", position: "absolute", textAlign: "center", 
             width: "100%", marginTop: "5px"}, 0,
            [emptyText]);
        parent.appendChild(this.emptyMessage);
        var tr = newnode("tr");
        for (var i = 0; i < this.columns.length; i++)
            tr.appendChild(newnode("td", this.columns[i].style,
                                   {className: "cell"}));
        this.rowTemplate = newnode("table", 0,
            {cellPadding: "0", cellSpacing: "0", className: "tr"},
            [this.getColGroup(), newnode("tbody", 0, 0, [tr])]);
    },
    
    /**
     * Adds a hover to the LiveGrid2.
     * This method should be called between getTable() and the first enable(). 
     * @param {Object} hover A DOM node which serves as a hover. It should be
     * attached directly to the body and have the CSS style
     * "position: absolute".
     * @param {Function} callback A callback function which is called when
     * the hover is shown. As parameters, it receives the ID of
     * the entry over which the mouse is hovering and whether the hover was
     * triggered manually.
     * @type {Object}
     * @return A new Hover object. The enable and disable methods of the hover
     * object will be called automatically by the LiveGrid2.
     */
    addHover: function(hover, callback) {
        this.hoverNode = hover;
        this.hoverCallback = callback;
		this.hover = new Hover(this.heightDiv, hover);
        var Self = this;
        this.hover.setSize = function(node) {
            Self.hoverSizeNode = node;
            return Hover.prototype.setSize.call(this, node);
        }
        this.hover.onShow = function(node, manual) {
            var i = node.oxRowIndex;
            if (i !== undefined)
                callback(Self.collection.objects[i], manual);
            else
                return false;
        }
        return this.hover;
    },
    
    /**
     * @private
     */
    getColGroup: function() {
        var colgroup = newnode("colgroup");
        for (var i = 0; i < this.columns.length; i++) {
            var w = this.columns[i].width;
            colgroup.appendChild(newnode("col", w ? {width: w} : 0));
        }
        return colgroup;
    },
    
    /**
     * Appends the contents either of the entire grid or of selected rows to
     * the specified node.
     * @param {Object} parent The node to append the data to.
     * @param {Function} cb The callback to call after all data is appended.
     * @param {Boolean} selected True if only selected rows should be appended.
     * Otherwise, all rows are appended.
     */
    print: function(parent, cb, selected) {
        var doc = parent.ownerDocument;
        var tr = doc.createElement("tr");
        for (var i = 0; i < this.columns.length; i++) {
            var th = doc.createElement("th");
            if (this.columns[i].i18n)
                th.appendChild(doc.createTextNode(_(this.columns[i].text) || "\u00a0"));
            else
                th.innerHTML = this.columns[i].text || "\u00a0";
            tr.appendChild(th);
        }
        var table = newnode("table", {width: "100%"}, {border: 1},
            [newnode("thead", 0, 0, [tr], doc)], doc);
        var trs = [];
        var Self = this;
        function set(n, obj) {
            trs[n] = doc.createElement("tr");
            for (var i = 0; i < Self.columns.length; i++) {
                var td = doc.createElement("td");
                trs[n].appendChild(td);
                var col = Self.columns[i];
                col.set(td, col.index ? obj[col.index] : obj, obj);
            }
        }
        function final_() {
            var tbody = newnode("tbody", 0, 0, 0, doc);
            for (var i = 0; i < trs.length; i++) 
                if (trs[i]) tbody.appendChild(trs[i]);
            table.appendChild(tbody);
            parent.appendChild(table);
            if(trs.length==0) {
                return; 
            }
            cb();
        }
		var selection = (selected) ? this.selection.getSelected() : this.collection.objects;
        OXCache.newRequest(null,this.module,{
                objects : selection,
                columns : this.listColumns()
            },null,function callback(data) {
				if (!data || !data.objects) {
					//return;
				}
				for(var i=0;i<data.objects.length;i++) {
					var obj=data.objects[i];
					var tr = doc.createElement("tr");
					for (var i2 = 0; i2 < Self.columns.length; i2++) {
                        var td = doc.createElement("td");
                        tr.appendChild(td);
					    var col = Self.columns[i2];
                        if(col.name.length==1) {
						  col.set(td, obj[col.name[0]], obj);
						} else {
					      col.set(td, obj, obj);
						}
  				    }
				    trs.push(tr); 
                 }
				 var tbody = newnode("tbody", 0, 0, 0, doc);
                 for (var i = 0; i < trs.length; i++) {
				 	if (trs[i]) {
						tbody.appendChild(trs[i]);
					}
				 }
                 table.appendChild(tbody);
                 parent.appendChild(table);
                 if(trs.length==0) {
                    //return; 
                 }
                 cb();
            }
		);    
    },
    
    /**
     * Displays the specified sort order as an arrow in the headers.
     * @param {String} id The column ID used for sorting, or null for no
     * sorting.
     * @param {String} order The sort order. One of "asc", "desc" or null for no
     * sorting.
     */
    setSort: function(name, order) {
        if (this.sort_id && name != this.sort_id) {
            var index = this.ids[this.sort_id];
			if(index || index === 0) {
			    this.arrows[index].src = getFullImgSrc("img/dummy.gif");
                LiveGrid2.setSortClass(this.outer_headers[index], false);
			}
		}
        var index = this.ids[name];
        if(index || index === 0) {
            this.arrows[index].src = 
                order == "asc" ? getFullImgSrc("img/arrows/sort_up.gif") :
                order == "desc" ? getFullImgSrc("img/arrows/sort_down.gif") :
                getFullImgSrc("img/dummy.gif"); // just in case
            LiveGrid2.setSortClass(this.outer_headers[index], true);
        }
		this.sort_id = name;
        this.sort_order = order;
        if (this.collection) this.sort(name, order);
    },
    
    /**
     * Callback function which changes the displayed data.
     * Should be overridden in each instance.
     * @param {String} id The column ID used for sorting, or null for no
     * sorting.
     * @param {String} order The sort order. Either "asc" or "desc".
     */
    sort: function() {
		if(!this.collection) {
			return
		}
		this.enable(this.collection.criteria, this.collection.search);
	},
    getSelectedIDs: function() {
        return this.selection.getSelected();
    },
    
    /**
     * Deletes items with specified object IDs from the collection and from
     * the selection. If the entire selection was deleted, the first unselected
     * item at or after the focus is selected.
     */
    deleteIDs: function(ids) {
		/*
        var sids = {};
        for (var i = 0; i < ids.length; i++)
            sids[this.storage.serialize(ids[i])] = true;
        while (this.storage.getSID(this.focus) in sids) this.focus++;
        var newID = this.storage.ids[this.focus];
        this.storage.removeIDs(ids);
        this.selection.deselectSIDs(sids);
        this.focus = newID ? this.storage.getIndex(newID) : 0;
        if (!this.selection.count)
            this.selection.click(this.focus);
        */
    },
    
    updateSelected: function(callback) {
        //if (this.collection)
            //this.storage.localUpdate(this.selection.getSelected(), callback);
    },

    /**
     * Initializes the LiveGrid2.
     * The DOM tree must have been created by {@link #getHeader} and
     * {@link #getTable}.
     * @see #getHTML
     */
	enable : function(criteria,search,force) {
		var Self=this;
		this.collectiondata = {
             "criteria" : criteria,
             columns : []
        };
		if (this.sort_id) {
            this.collectiondata.order = [{ sort : this.sort_id,
                                           order : this.sort_order }];
		}
		if(search) {
			this.collectiondata["search"]=search;
		}
		if(this.uniqueName) {
		    OXCache.unregister(this.uniqueName);
            delete this.uniqueName;
        }
		if(this.listRequest) {
            OXCache.unregister(this.listRequest.uniqueName);
            delete this.listRequest;
        }   
        if(!this.collection && this.collection !== false) {
            this.collection = false;
			for (var i in Self.arrows) {
                addDOMEvent(Self.outer_headers[i], "click", Self.arrow_events[i]);
			}
            addDOMEvent(Self.parent, "scroll", Self.scroll_cb);
            addDOMEvent(Self.parent, "keydown", Self.keydown_cb);
            addDOMEvent(Self.parent, "keypress", Self.keypress_cb);
            addDOMEvent(Self.parent, "mousedown", Self.mousedown_cb);
            addDOMEvent(Self.parent, "click", Self.click_cb);
            // bind double click via jQuery (due to bug #14560)
            jQuery(this.parent).bind("dblclick", this.dblclick_cb);
            addDOMEvent(Self.parent, "focus", Self.focus_cb);
            addDOMEvent(Self.parent, "blur", Self.blur_cb);
            if (this.contextmenu)
                addDOMEvent(this.parent, "contextmenu", this.ctxtmenu_cb);
            if (Self.hover && !Self.disablehover) Self.hover.enable();
            resizeEvents.register("Resized", Self.scroll_cb);
            register("LanguageChanged",Self.language_cb);
            if (Self.dndModule && (Self.dndModule != "links")) {
                var disabled;
                var validfield;
                switch(Self.dndModule) {
                    case "calendar": disabled=calendardefaultdisabled;  validfield="title"; break;
                    case "tasks":   disabled=taskdefaultdisabled; validfield="title"; break;
                    case "contacts": disabled=contactdefaultdisabled; validfield="display_name"; break;
                    case "infostore": disabled=infostoredefaultdisabled; validfield="title"; break;
                    case "mail": disabled=maildefaultdisabled; validfield="subject"; break;
                }
                Self.dragEvent = registerSource(Self.heightDiv, Self.dndModule, function() 
                {
                    var myelements=clone(Self.selection.getSelected());
                    objMoveCopy.begin(Self.dndModule,myelements);
                    return objMoveCopy;
                },null,null,disabled,defaultdisabledremove);
            }
        }
        var oldFocus = 0, fallbackSelection;
        var returnObject=OXCache.newRequest(null, this.module,
            this.collectiondata, {
                before: function() {
                    oldFocus = Self.focus;
                    if (!Self.collection || !Self.selection.collection) return;
                    while (Self.selection.get(oldFocus)) oldFocus++;
                    fallbackSelection = Self.collection.objects[oldFocus];
                },
                fn: function modifiedCallback(data) {
                    Self.collection=data;
                    Self.rowCount = data.objects.length;
                    var index;
                    if (fallbackSelection) {
                        index = Self.collection.getIndex(fallbackSelection) - 1;
                    }
                    if (isNaN(index)) {
                        index = Math.min(oldFocus, Self.rowCount - 1);
                    }
                    if (Self.selection.setCollection(Self.collection,
                            Self.selection.count ? index : undefined))
                    {
                        Self.focus = index;
                    }
                    Self.change_cb(0,data.objects.length); 
                    Self.events.trigger("Changed", data, true);
                }
            },
			function callback(data) {
				Self.collection=data;
	            Self.rowCount = data.objects.length;
	            Self.selection.setCollection(Self.collection, Self.selection.selectedIndex);
	            Self.scroll(true);
				Self.events.trigger("Changed", data, false);
			},force);
		this.uniqueName=returnObject.uniqueName;
		Self.selection.events.register("Selected", Self.selected_cb);
	},
    disable: function() {
        if (!this.collection) return;
        this.selection.events.unregister("Selected", this.selected_cb);
        if(this.uniqueName) {
			OXCache.unregister(this.uniqueName);
            delete this.uniqueName;
		}
		if(this.listRequest) {
            OXCache.unregister(this.listRequest.uniqueName);
            delete this.listRequest;
        }
		this.collection = null;
        this.selection.setCollection(null);
        for (var i in this.arrows)
            removeDOMEvent(this.outer_headers[i], "click", this.arrow_events[i]);
        removeDOMEvent(this.parent, "scroll", this.scroll_cb);
        removeDOMEvent(this.parent, "keydown", this.keydown_cb);
        removeDOMEvent(this.parent, "keypress", this.keypress_cb);
        removeDOMEvent(this.parent, "mousedown", this.mousedown_cb);
        removeDOMEvent(this.parent, "click", this.click_cb);
        jQuery(this.parent).unbind("dblclick", this.dblclick_cb);
        removeDOMEvent(this.parent, "focus", this.focus_cb);
        removeDOMEvent(this.parent, "blur", this.blur_cb);
        if (this.contextmenu)
            removeDOMEvent(this.parent, "contextmenu", this.ctxtmenu_cb);
        unregister("LanguageChanged",this.language_cb);
        if (this.hover) this.hover.disable();
        resizeEvents.unregister("Resized", this.scroll_cb);
        if (this.dndModule) {
            unregisterSource(this.parent, this.dragEvent);
        }
    },
    
    /**
     * @private
     */
    createRow: function() {
        var last = this.columns.length - 1;
        var row = {tr: this.rowTemplate.cloneNode(true),
                   cells: new Array(this.columns.length)};
        for (var i = 0, cell = row.tr.lastChild.firstChild.firstChild; cell;
             i++, cell = cell.nextSibling)
        {
            row.cells[i] = cell;
            this.columns[i].clear(cell);
        }
        this.heightDiv.appendChild(row.tr);
        if (!this.rowHeight) this.rowHeight = row.tr.offsetHeight + this.rowMargin;
        return row;
    },

    /**
     * To be removed.
     * @private
     */
    updateStyles: function() {
    	// return if the selection has no collection
    	if (!this.selection.collection) return;
        for (var i in this.rows) {
            var newclass = this.selection.get(i) ? "tr selected" : i%2 ? "tr_h " : "tr ";
            if (focusedElement == this.parent && i == this.focus)
                newclass += "focused";
            var tr = this.rows[i].tr;
            if (tr.className != newclass) tr.className = newclass;
        }
    },

    /**
     * @private
     */
    deleteRow: function(index) {
        var row = this.rows[index];
        row.tr.style.display = "none";
        this.rowcache.push(row);
        delete this.rows[index];
        delete this.unfetched[index];
    },

    /**
     * @private
     */
    insertRow: function(index) {
        var row = this.rows[index] =
            this.rowcache.length ? this.rowcache.pop() : this.createRow();
        row.tr.className = this.selection.get(index) ? "tr selected" : index%2 ? "tr_h" : "tr";
        row.tr.style.display = "";
        row.tr.style.top = (index * this.rowHeight) + "px";
        row.tr.oxRowIndex = index;
        this.unfetched[index] = true;
    },

    /**
     * @private
     */
    scroll: function(fullRedraw) {
        if (fullRedraw) this.fullRedraw = true;
        if (this.scrollTimeout) {
			clearTimeout(this.scrollTimeout);
		}
        var Self = this;
        this.scrollTimeout = setTimeout(function() {
            Self.scrollTimeout = null;
            Self.doScroll();
        }, 0);
    },
    
    /**
     * @private
     */
    calculateRowHeight: function() {
        var row = this.createRow();
        row.tr.style.display = "none";
        this.rowcache.push();
    },

    /**
     * @private
     */
    doScroll: function() {
		var Self=this;       
        if (!this.rowHeight) this.calculateRowHeight();
        this.height = Math.floor(this.parent.clientHeight / this.rowHeight);
        var len = this.collection ? this.collection.objects.length : 0;
        var newHeight = (len * this.rowHeight) + "px"
        var hDivStyle = this.heightDiv.style;
        if (newHeight != hDivStyle.height) hDivStyle.height = newHeight;
        var newruWidth = this.parent.offsetWidth - this.parent.clientWidth;
        if (newruWidth > 0 && newruWidth != this.ruWidth) {
            this.ruCorner.style.width = newruWidth + "px";
            this.ruWidth = newruWidth;
        }
        var Self = this;
        if (this.heightDiv.offsetWidth != this.parent.clientWidth)
            hDivStyle.width = Self.parent.clientWidth + "px";
        var scrolltop = this.parent.scrollTop;
        var from2 = Math.max(0, Math.floor(                           scrolltop  / this.rowHeight / this.granularity) * this.granularity - this.preview);
        var to2 = Math.min(len, Math.ceil((this.parent.clientHeight + scrolltop) / this.rowHeight / this.granularity) * this.granularity + this.preview);
		if (this.fullRedraw || from2 != this.from || to2 != this.to) {
            if (this.fullRedraw || from2 > this.to || this.from > to2) {
                for (var i = this.from; i < this.to; i++) this.deleteRow(i);
                for (var i = from2; i < to2; i++) this.insertRow(i);
            } else {
                for (var i = this.from; i < from2; i++) this.deleteRow(i);
                for (var i = to2; i < this.to; i++) this.deleteRow(i);
                for (var i = from2; i < this.from; i++) this.insertRow(i);
                for (var i = this.to; i < to2; i++) this.insertRow(i);
            }
            this.from = from2;
            this.to = to2;
        }
        this.fullRedraw = false;
        this.emptyMessage.style.display = (!this.collection || len) ? "none"
                                                                    : "block";
        var ids = [];
        for (var i in this.unfetched) ids.push(this.collection.objects[i]);
        if (ids.length) {        	
            if(this.listRequest) {
				OXCache.unregister(this.listRequest.uniqueName);
				delete this.listRequest;
			}
            var collection = this.collection;
		    this.listRequest=OXCache.newRequest(null,this.module,{
                objects : ids,
                columns : this.listColumns()
            },null,function cb(data) {
                for(var i=0;i<data.objects.length;i++) {
                    var objIndex=collection.map_objects.get(data.objects[i]);
                    if(!Self.rows[objIndex-1]) return;
                    delete Self.unfetched[objIndex-1];
                    var cellrow = Self.rows[objIndex-1].cells;
                    for (var i2 in cellrow) {
                        var col = Self.columns[i2];
                        if(col.name.length==1 && col.name[0].indexOf("/") < 0) {
                            col.set(cellrow[i2], data.objects[i][col.name[0]],
                                data.objects, Self);
                        } else {
                            col.set(cellrow[i2], data.objects[i], data.objects,
                                Self);
                        }
                    }
                }
                delete Self.listRequest;
            },false);
			if(this.listRequest) {
				for(var i in this.listRequest.clear) {
                    var objIndex=this.collection.map_objects.get(this.listRequest.clear[i]);
                    var cellrow = Self.rows[objIndex-1].cells;
                    for (var i in cellrow) Self.columns[i].clear(cellrow[i]);
                }
				for(var i in this.listRequest.set) {
                	var objIndex=this.collection.map_objects.get(this.listRequest.set[i]);
					var cellrow = Self.rows[objIndex-1].cells;
                    for (var i2 in cellrow) {
                        var col = Self.columns[i2];
                        if(col.name.length==1 && col.name[0].indexOf("/") < 0) {
                            col.set(cellrow[i2],
                                this.listRequest.set[i][col.name[0]], null,
                                Self);
                        } else {
                            col.set(cellrow[i2], this.listRequest.set[i], null,
                                  Self);
                        }
                    }
                }
            }
        }
    },
    
    /**
     * @private
     */
    scrollTo: function(y) {
        var parent = this.parent;
        setTimeout(function() { parent.scrollTop = y; }, 0);
    },
    
    /**
     * Scrolls if necessary to display the currently focused row.
     */
    showFocus: function() {
        var scrolltop = this.parent.scrollTop;
        if (!this.rowHeight) this.calculateRowHeight();
        if (this.focus < scrolltop / this.rowHeight)
            this.scrollTo(this.focus * this.rowHeight);
        else {
            var height = this.parent.clientHeight;
            if (this.focus + 1 > (scrolltop + height) / this.rowHeight)
                this.scrollTo((this.focus + 1) * this.rowHeight - height);
        }
    },

    /**
     * @private
     */
    key: function(e) {
        if (!this.collection) return;
        switch (e.keyCode || e.which) {
        case 13: // enter
            this.events.trigger("Activated", this.selection.getSelected());
            break;
        case 32: // spacebar
            this.selection.click(this.focus, e);
            this.showFocus();
            break;
        case 46: // delete
        case 108: // numpad delete
            this.events.trigger("Delete", this.selection.getSelected());
            break;
        case 38: // arrow up
            if (this.focus > 0) this.focus--;
            if (!(Mac ? e.metaKey : e.ctrlKey))
                this.selection.click(this.focus, e);
            this.showFocus();
            break;
        case 40: // arrow down
            if (this.focus < this.rowCount - 1) this.focus++;
            if (!(Mac ? e.metaKey : e.ctrlKey)) {
                this.selection.click(this.focus, e);
            }
            this.showFocus();
            break;
        case 33: // page up
            this.focus -= this.height;
            if (this.focus < 0) this.focus = 0;
            if (!(Mac ? e.metaKey : e.ctrlKey))
                this.selection.click(this.focus, e);
            this.showFocus();
            break;
        case 34: // page down
            this.focus += this.height;
            if (this.focus >= this.rowCount) this.focus = this.rowCount - 1;
            if (!(Mac ? e.metaKey : e.ctrlKey))
                this.selection.click(this.focus, e);
            this.showFocus();
            break;
        case 35: // end
            this.focus = this.rowCount - 1;
            if (!(Mac ? e.metaKey : e.ctrlKey))
                this.selection.click(this.focus, e);
            this.showFocus();
            break;
        case 36: // home
            this.focus = 0;
            if (!(Mac ? e.metaKey : e.ctrlKey))
                this.selection.click(this.focus, e);
            this.showFocus();
            break;
        case 65: // A
        case 97: // a
            if (e.ctrlKey)
                this.selection.select(0, this.collection.objects.length);
            break;
        default:
            return;
        }
        this.updateStyles();
        stopEvent(e);
        return false;
    },
    
    /**
     * @private
     */
    mousedown: function(e) {
        this.clickprocessed = false;
        var focus = getRowIndex(e);
        // might be undefined (timing issue): this.collection.objects - so check that
        if (this.collection.objects && this.collection.objects.length < this.height) {
            if (focus === undefined) {
                focus = -1;
            } else {
                this.focus = focus;
            }
            if (!this.selection.get(focus)) {
                this.clickprocessed=true;
                this.selection.click(focus, e);
            }
        } else if (focus !== undefined) {
            this.focus = focus;
            if (!this.selection.get(focus)) {
                this.clickprocessed=true;
                this.selection.click(focus, e);
            }
        }
        setFocus(this.parent);
    },
    
    /**
     * @private
     */
    ctxtmenu: function(e) {
        stopEvent(e);
        this.contextmenu.display(e.clientX, e.clientY, this.selection);
    },
    
    /**
     * @private
     */
    click: function(e) {
        if (this.clickprocessed) {
            return;
        }
        var focus = getRowIndex(e);
        // might be undefined (timing issue): this.collection.objects - so check that
        if (this.collection.objects && this.collection.objects.length < this.height) {
            if (focus === undefined) focus = -1; else this.focus = focus;
            this.selection.click(focus, e);
        } else if (focus !== undefined) {
            this.focus = focus;
            this.selection.click(focus, e);
        }
        cancelDefault(e);
        if (!IE6 && corewindow.hasFocus) setFocus(this.parent);
    },
    
    /**
     * @private
     */
    dblclick: function(e) {
        if (!this.collection) return;
        this.focus = getRowIndex(e) || 0;
        this.events.trigger("Activated", this.selection.getSelected());
    }
};

LiveGrid2.setSortClass = classNameSetter("livegrid-th-sorted");

/**
 * Default set function for use in the second parameter of the constructor.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified content value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} text The text to set in the DIV element.
 */
LiveGrid2.defaultSet = function(div, text) {
    text = text || "\u00a0";
    if (div.firstChild)
        div.firstChild.data = text;
    else
        div.appendChild(document.createTextNode(text));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid2.dateSet = function(div, date) {
    var dateString = "\u00a0";
    if (null != date) dateString = formatDate(date, "date");
    if (div.firstChild)
        div.firstChild.data = dateString;
    else
        div.appendChild(document.createTextNode(dateString));
};

/**
 * Set function for use in formatting dates in the live grid.
 * This function adds a text node in the specified DIV element and sets text
 * to the specified date value.
 * @param {Object} div The HTML DIV element to fill with content.
 * @param {String} utc The utc to format in the DIV element.
 */
LiveGrid2.dateSetTime = function(div, date) {
    if (div.firstChild)
        div.firstChild.data = formatDate(date, "datetime");
    else
        div.appendChild(document.createTextNode(formatDate(date, "datetime")));
};

/**
 * Creates a clear function for use in the second parameter of the constructor.
 * This function creates a function which sets all DIV elements of the LiveGrid2
 * to the text which is specified as parameter to this function.
 * @param {String} text The text to set in DIV elements.
 * @type Function
 * @return A function which takes a HTML DIV element as parameter, adds a text
 * node to it, and sets the text to a constant value.
 */
LiveGrid2.makeClear = function(text) {
    return function(div) {
        if (div.firstChild)
            div.firstChild.data = text;
        else
            div.appendChild(document.createTextNode(text));
    };
};




fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Suphi Basdemir <suphi.basdemir@open-xchange.com>
 * 
 */

var globalclickdisable2=true;
var activeComboBox = null;

function changePopUpMenuText3(obj, n, noevent)
{
    var myValues = [];
    myValues = obj.elements[n];
    
    if (myValues.callback.nodeType || typeof myValues.callback == "function") {
        obj.currentI18nNode.callback = myValues.callback;
    } else {
        obj.currentI18nNode.callback = function() {
            return String(myValues.callback);
        };
    }
    obj.currentI18nNode.update();
            
    obj.selected = n;
     var key = myValues.key;
    
    if (obj.targetDiv) {
        if(obj.form) {
            obj.targetDiv.value = key;
        } else {
            if(!obj.valueIsHTML) {
                if(obj.targetDiv && obj.targetDiv.firstChild) {
                    obj.targetDiv.firstChild.data = key;
                }
            }
            else {
                removeChildNodes(obj.targetDiv);
                if(key) {
                    obj.targetDiv.appendChild(key);
                }
            }
                
        }        
    }
    if (obj.onChange && !noevent) {
        obj.onChange(key, obj.currentResult);
    }
        
    obj.currentResult = key;
}

function createComboBoxOverlay(scope) {
    // overlay - combo-box may block - not unusual
    return jQuery("<div/>")
        .addClass("oxStretch")
        .css({
            display: "none",
            opacity: 0.01,
            zIndex: 40000,
            backgroundColor: "white"
        })
        .bind("click", function (e) {
            scope.overlay.hide();
            showHideBox3(scope, e);
        })
        .appendTo(document.body);
}

/**
 * Make ComboBox
 * @param {object} current Window where the combobox is called
 * @param {String} id of rootDiv, to append the combobox
 * @param {String} width from the combobox
 * @param {int} index of selected item from the elements
 * @param {boolean} default Style, should be used ore not
 * @param {String} eventname to trigger an event onChange
 * @param {int} scrollbar should be shown after n items
 * example:
 * new ComboBox3(window, "start_time", "15em", myElements, 0, false, changetime, 6);
 */

function ComboBox3(window, elementid, width, selected, style, eventname, maxscrollbar,eventParam) {
    return new ComboBox3.impl(window, ($ALL(elementid) || newnode("div")), width, selected, style,
        maxscrollbar,
        function(key) { triggerEvent(eventname, key, eventParam); } );
}

ComboBox3.impl = function (window, rootDiv, width, selected, style, maxscrollbar, onchange, onshow, onhide) {
    this.window = window;
    this.selected = selected;
    this.elements = [];
    this.width=width;
    this.maxscrollbar=maxscrollbar;
    this.style = style;
    this.targetDiv = null;
    this.valueIsHTML = false; //the value is a node
    this.form = false;
    this.viewValue = false;
    this.disabled = false;        
    this.rowHeight;    
    this.childOffsetTop;
    var Self = this;
    this.rootDiv = rootDiv;
    this.rootDiv.tabIndex = 0;
    this.rootDiv.style.outline = "none";
    this.keydown_cb = function(e) {
        return Self.key(e);
    };
    addDOMEvent(this.rootDiv, "keydown", this.keydown_cb);
    this.rootDiv.innerHTML = "";
    this.headTable = newnode('table',0,{
                     cellPadding: 0, cellSpacing: 0, border: 0});
    this.headTable.style.tableLayout="fixed";
    this.headTableBody = document.createElement("tbody");
    this.headTableRow = document.createElement("tr");
    this.headTableColumn1 = document.createElement("td");    
    this.headTableColumn1.setAttribute("nowrap","nowrap");
    this.headTableColumn1.style.whiteSpace = "nowrap";
    this.headTableColumn1div = document.createElement("div");    
    this.headTableColumn1.appendChild(this.headTableColumn1div);
    this.headTableColumn1div.style.width = "92%";    
    this.headTableColumn1div.style.paddingRight = "3px";        
    this.headTableColumn1div.style.overflow = "hidden";    
    this.headTableColumn2 = document.createElement("td");
    this.headTableColumn2.setAttribute("width","14");
    this.myboxComboDiv = document.createElement("div");
    jQuery(this.myboxComboDiv).click(function(evt) {
    	// handle click on combo layer
        showHideBox3(Self, evt);
        return false;
    });
    this.keySearch = {};
    
    if(this.style) {
        this.headTable.className="smallpopupborder border-color-design smallpopupnotactive background-color-content font-color-default";
        this.headTableColumn1.className="smallpopupborderright border-color-design comboboxColumn1 cursorPointer";
        this.headTableColumn2.className="comboboxColumn2 borderLeftDefault border-color-design";
    }
    this.rootDiv.style.width=this.width;
    this.rootDiv.className="dropdownbox";
    
    this.currentValues = [];
    
    this.currentResult = "";
    
    this.onChange = onchange;
    this.onShow = onshow;
    this.onHide = onhide;
        
    if(maxscrollbar) {  
        this.scrollbar=maxscrollbar;
    } else { 
        this.scrollbar=null;
    }
        
    this.setFirstDiv();
    
    // overlay
    this.overlay = createComboBoxOverlay(this);
};

ComboBox3.impl.prototype = {
    sort: function () {
        function fn(a,b) {
            if(a.value < b.value) {
                return -1;
            } else {
                return 1;
            }
        }
        this.elements.sort(fn);
    },
    sorti18nValue: function () {
        function fn(a,b) {
            //if(a.i18nNode.node.nodeValue < b.i18nNode.node.nodeValue)
            if(a.key == "" ) {
                return -1;
            } else if(b.key == "" ) { 
                return 1;
            } else {
                return a.i18nNode.node.nodeValue.localeCompare( b.i18nNode.node.nodeValue ) ;
            }
        }
        this.elements.sort(fn);
    },
    sortKeys: function () {
        function fn(a,b) {
            if(a.key < b.key) {
                return -1;
            } else {
                return 1;
            }
        }
        this.elements.sort(fn);
    },
    getValue : function () {
        this.currentValues = this.elements[this.selected];
        this.currentValue = this.currentValues.value;
        return this.currentValue;
    },
    setEnabled : function (value) {
        //TODO DISABLED;
    },
    getKey : function () {
        this.currentValues = this.elements[this.selected];
        this.currentResult = this.currentValues.key;
        return this.currentResult;
    }, 
    getSelectedIndex : function () {
        return this.selected;
    },
    setSelectedIndex : function (n, noevent) {
        changePopUpMenuText3(this, n, noevent);
    },
    disable : function () {
        var Self = this;
        Self.headTable.className = Self.headTable.className.replace(/font-color-default/, "font-color-disabled");
        Self.disabled = true;
        Self.headTableColumn2.style.cursor = "default";
        //Self.window.removeDOMEvent(Self.headTable,"click", Self.clickOnCombobox);
    },
    enable : function () {
        var Self = this;
        Self.disabled = false;            
        Self.headTable.className = Self.headTable.className.replace(/font-color-disabled/, "font-color-default");
        Self.headTableColumn2.style.cursor = "pointer";
        //Self.window.addDOMEvent(Self.headTable,"click", Self.clickOnCombobox);  
    },
    setKey : function (key,noevent) {
        var myElValues = [];
        var newIndex = key;
        var bFound = false;
        for(i=0; i<this.elements.length;i++) {
            myElValues = this.elements[i];
                    if (key == myElValues.key) {
                        key = myElValues.key;
                        newIndex = i;
                        bFound = true;
                        break;
                    }               
        }
        if(!bFound) {
            return false;
        }
        if(this.elements[newIndex]) {
            changePopUpMenuText3(
                    this,
                    newIndex,
                    noevent
            );
        }                    
        return true;        
    },    
    setValue : function (value,noevent) {
        var myElValues = [];
        var newIndex = value;
        var bFound = false;
        for(i=0; i<this.elements.length;i++) {
            myElValues = this.elements[i];
            
            if(myElValues.i18nNode && myElValues.i18nNode.node && myElValues.i18nNode.node.nodeValue) {
                myi18nValue = myElValues.i18nNode.node.nodeValue;
            } else {
                myi18nValue = this.elements[i];
            }
                
             if (value == myElValues.value || value == myi18nValue) {
                 value = myElValues.value;
                 newIndex = i;
                 bFound = true;
                 break;
             }         
        }
        if(!bFound) {
            return false;
        }
        if(this.elements[newIndex]) {
            changePopUpMenuText3(
                    this,
                    newIndex,
                    noevent
            );
        }                    
        return true;        
    },    
    setUnknownKey : function(key) {                    
        this.targetDiv.firstChild.data = key;
    },
    scrollToSelected: function (tmpIndex) {
        if (this.scrollbar) {
            // vars
            var index, node, top;
            // get index
            index = tmpIndex !== undefined ? tmpIndex : this.selected;
            // get position & scroll
            node = jQuery(this.myboxComboDiv);
            top = (node.scrollTop() + node.children().eq(index).position().top) >> 0;
            node.scrollTop(top);
        }
    },
    
    key: function(e) {
        switch (e.keyCode || e.which) {
        case 13: // enter
            break;
        case 32: // spacebar
            break;
        case 38: // arrow up
            break;
        case 40: // arrow down
            break;
        case 65: // A
        case 97: // a        
            if (this.keySearch["a"]) {
                this.scrollToSelected(this.keySearch["a"][0]);
            }
            break;
        case 66: // B
        case 98: // b    
            if (this.keySearch["b"]) {    
                this.scrollToSelected(this.keySearch["b"][0]);
            }
            break;
        case 67: // C
        case 99: // c    
            if (this.keySearch["c"]) { 
                this.scrollToSelected(this.keySearch["c"][0]);
            }
            break;
        case 68: // D
        case 100: // d        
            if (this.keySearch["d"]) {
                this.scrollToSelected(this.keySearch["d"][0]);
            }
            break;
        case 69: // E
        case 101: // e        
            if (this.keySearch["e"]) {
                this.scrollToSelected(this.keySearch["e"][0]);
            }
            break;
        case 70: // F
        case 102: // f        
            if (this.keySearch["f"]) {
                this.scrollToSelected(this.keySearch["f"][0]);
            }
            break;
        case 71: // G
        case 103: // g        
            if (this.keySearch["g"]) {
                this.scrollToSelected(this.keySearch["g"][0]);
            }
            break;
        case 72: // H
        case 104: // h        
            if (this.keySearch["h"]) {
                this.scrollToSelected(this.keySearch["h"][0]);
            }
            break;
        case 73: // I
        case 105: // i        
            if (this.keySearch["i"]) {
                this.scrollToSelected(this.keySearch["i"][0]);
            }
            break;
        case 74: // J
        case 106: // j        
            if (this.keySearch["j"]) {
                this.scrollToSelected(this.keySearch["j"][0]);
            }
            break;
        case 75: // K
        case 107: // k        
            if (this.keySearch["k"]) {
                this.scrollToSelected(this.keySearch["k"][0]);
            }
            break;
        case 76: // L
        case 108: // l        
            if (this.keySearch["l"]) {
                this.scrollToSelected(this.keySearch["l"][0]);
            }
            break;
        case 77: // M
        case 109: // m        
            if (this.keySearch["m"]) {
                this.scrollToSelected(this.keySearch["m"][0]);
            }
            break;
        case 78: // N
        case 110: // n        
            if (this.keySearch["n"]) {
                this.scrollToSelected(this.keySearch["n"][0]);
            }
            break;
        case 79: // O
        case 111: // o    
            if (this.keySearch["o"]) {
                this.scrollToSelected(this.keySearch["o"][0]);
            }
            break;
        case 80: // P
        case 112: // p        
            if (this.keySearch["p"]) {
                this.scrollToSelected(this.keySearch["p"][0]);
            }
            break;
        case 81: // Q
        case 113: // q        
            if (this.keySearch["q"]) {
                this.scrollToSelected(this.keySearch["q"][0]);
            }
            break;
        case 82: // R
        case 114: // r        
            if (this.keySearch["r"]) {
                this.scrollToSelected(this.keySearch["r"][0]);
            }
            break;
        case 83: // S
        case 115: // s        
            if (this.keySearch["s"]) {
                this.scrollToSelected(this.keySearch["s"][0]);
            }
            break;
        case 84: // T
        case 116: // t        
            if (this.keySearch["t"]) {
                this.scrollToSelected(this.keySearch["t"][0]);
            }
            break;
        case 85: // U
        case 117: // u        
            if (this.keySearch["u"]) {
                this.scrollToSelected(this.keySearch["u"][0]);
            }
            break;
        case 86: // V
        case 118: // v        
            if (this.keySearch["v"]) {
                this.scrollToSelected(this.keySearch["v"][0]);
            }
            break;
        case 87: // W
        case 119: // w        
            if (this.keySearch["w"]) {
                this.scrollToSelected(this.keySearch["w"][0]);
            }
            break;
        case 88: // X
        case 120: // x        
            if (this.keySearch["x"]) {
                this.scrollToSelected(this.keySearch["x"][0]);
            }
            break;
        case 89: // Y
        case 121: // y        
            if (this.keySearch["y"]) {
                this.scrollToSelected(this.keySearch["y"][0]);
            }
            break;
        case 90: // Z
        case 122: // z        
            if (this.keySearch["z"]) {
                this.scrollToSelected(this.keySearch["z"][0]);
            }
            break;
        default:
            return;
        }
        stopEvent(e);
        return false;
    },
    
    setFirstDiv: function () {
        var Self = this;
                        
        Self.clickOnCombobox = function (e) { 
            if(!Self.disabled) {
                showHideBox3(Self,e);
                cancelBubbling(e);
            }
            cancelDefault(e);
        };
        
        Self.headTable.style.width=this.width;    
        setTimeout(function () {
            var tmpW = Self.headTable.offsetWidth - 16;
                
            Self.headTableColumn2.style.width="16px";
            Self.headTableColumn2.style.textAlign="center";
        
            Self.image=document.createElement("img");
            Self.image.className="cursorPointer";
            Self.image.setAttribute("src",getFullImgSrc("img/arrows/arrow_white_down.png"));
            Self.headTableColumn2.appendChild(Self.image);
            Self.headTableRow.appendChild(Self.headTableColumn1);    
            Self.headTableRow.appendChild(Self.headTableColumn2);
            Self.headTableBody.appendChild(Self.headTableRow);
            Self.headTable.appendChild(Self.headTableBody);
            
            Self.window.addDOMEvent(Self.headTable,
                "click", Self.clickOnCombobox);  
                
               Self.window.addDOMEvent(Self.headTable,
                "mouseover", function() { globalclickdisable2=true; }); 
                
               Self.window.addDOMEvent(Self.headTable,
                "mouseout", function() { globalclickdisable2=false; });  
        }, 0);        
        
        Self.rootDiv.appendChild(Self.headTable);
        
    },
    
    clickOnCombobox: function (e) { 
        showHideBox3(this,e);
    },
    
    addElement: function (value, key, callback) {
        if (typeof value == "string") {
            if(value.length > 0) {
                var indexKeySearchSingle = value.substr(0,1).toLowerCase();
                if (typeof this.keySearch[indexKeySearchSingle] == "undefined") {
                    this.keySearch[indexKeySearchSingle] = [];
                }
                this.keySearch[indexKeySearchSingle].push(this.elements.length);
            }
            /*
             * for later implemantation
            if(key.length > 1) {
                var indexKeySearchDuo = key.substr(0,2).toLowerCase();
                if(typeof this.keySearch[indexKeySearchDuo] == "undefined") this.keySearch[indexKeySearchDuo] = new Array();
                this.keySearch[indexKeySearchDuo].push(this.elements.length);
            }
            */
        }
        this.elements.push({
            value: value,
            key: key,
            callback: callback || value
        });
    },
    
    getElement: function (key) {
        for (var i in this.elements) {
            if (this.elements[i].key == key)  {
                return this.elements[i];
            }
        }
    },
    
    getDomNode: function (sortedAfterTranslation) {    
        
        function fakeI18n(callback) {
            this.callback = callback;
            this.node = newnode("div",0,0,[callback.cloneNode(true)]);
            this.update = function() {
                removeChildNodes(this.node);
                this.node.appendChild(this.callback.cloneNode(true));
            };
        }
        
        while (this.myboxComboDiv.childNodes.length > 0) {
            this.myboxComboDiv.removeChild(this.myboxComboDiv.firstChild);
        }

        this.currentValues = this.elements[this.selected];
        this.currentValue = this.currentValues.value;
        this.currentResult = this.currentValues.key;    
        while (this.headTableColumn1div.childNodes.length > 0) {
            this.headTableColumn1div.removeChild(this.headTableColumn1div.firstChild);
        }
        if (this.currentValues.value.nodeType) {
            this.currentI18nNode = new fakeI18n(this.currentValues.callback);            
        } else {
            this.currentI18nNode = new I18nNode(this.currentValues.callback);
        }
        this.headTableColumn1div.appendChild(this.currentI18nNode.node);
        var Self = this;            
        Self.myboxComboDiv.className="smallpopupborder smallpopuplist background-color-content font-weight-default font-color-default border-color-design";
        Self.myboxComboDiv.style.display="none";
        Self.myboxComboDiv.style.textAlign="left";
        Self.myboxComboDiv.style.whiteSpace = "nowrap";    
               
        for(var el1 in Self.elements) {
            (function (n) {
                var myValues = [];                                     
                   myValues = Self.elements[n];
                   if (myValues.value.nodeType) {
                       myValues.i18nNode = new fakeI18n(myValues.callback);
                   } else {
                       myValues.i18nNode = new I18nNode(myValues.callback);
                   }
            })(el1);
        }
        
        if (sortedAfterTranslation) {
            this.sorti18nValue();    
               Self.keySearch = {};
               Self.indexForKeySearch = 0;        
        }
        
        
        for(var el in Self.elements)
        {
            (function (n) {
                var childDiv = null;
                var myValues = [];
                childDiv = newnode("div",0,{className:"height16 background-color-content cursorDefault"});
                //childDiv.style.overflow="hidden";
                childDiv.style.padding="3px 0 0 4px";
                Self.window.addDOMEvent(childDiv,
                    "click", function() { 
                             changePopUpMenuText3(Self,n);
                             Self.rootDiv.blur(); 
                         }
                   );
                   
                Self.window.addDOMEvent(childDiv,
                    "mouseover", function() { changePopUpMenuStatus3(childDiv, true, Self); });
                    
                Self.window.addDOMEvent(childDiv,
                    "mouseout", function() { changePopUpMenuStatus3(childDiv, false, Self); });
                                 
                    myValues = Self.elements[n];
                    
                    if (sortedAfterTranslation) {
                        if(typeof myValues.i18nNode.node.nodeValue == "string" && myValues.value != "please select") {
                            if(myValues.i18nNode.node.nodeValue.length > 0) {
                                var indexKeySearchSingle = myValues.i18nNode.node.nodeValue.substr(0,1).toLowerCase();
                                if (typeof Self.keySearch[indexKeySearchSingle] == "undefined") {
                                    Self.keySearch[indexKeySearchSingle] = [];
                                }
                                Self.keySearch[indexKeySearchSingle].push(Self.indexForKeySearch);
                            }
                        }
                        Self.indexForKeySearch++;
                    }
                    
                    if (Self.viewValue) {
                        if (myValues.key != "") {
                            childDiv.className = "height16 font-color-default background-color-content";
                        } else {
                            childDiv.className = "height16 font-color-disabled background-color-content";
                        }
                   }
                   if (!myValues.i18nNode) {
                       if (myValues.value.nodeType) {
                           myValues.i18nNode = new fakeI18n(myValues.callback);
                       } else {                           
                           myValues.i18nNode = new I18nNode(myValues.callback);
                       }
                   }
                   childDiv.appendChild(myValues.i18nNode.node);
                
                if (!Self.rowHeight) {
                    var tmpnode=childDiv.cloneNode(true);
                    tmpnode.style.display="block";
                    tmpnode.style.zIndex=-1;
                    body.appendChild(tmpnode);                
                    Self.rowHeight=tmpnode.offsetHeight;
                    body.removeChild(tmpnode);
                }
                    
                Self.myboxComboDiv.appendChild(childDiv);
            })(el);
        }
        if(this.scrollbar && this.scrollbar != "" && typeof this.scrollbar != "undefined") {
            var tempHeight = Math.floor(this.scrollbar*this.rowHeight);
            Self.myboxComboDiv.style.height=tempHeight+"px";
        }
        
        document.body.appendChild(Self.myboxComboDiv);
        
        if (this.maxscrollbar) {
            Self.myboxComboDiv.style.overflowX="hidden";
        } else {
            Self.myboxComboDiv.style.overflow="hidden";
        }
        
        return Self.rootDiv;
    }
};

function ComboBoxSpecial(window, element, width, selected, style, eventname, maxscrollbar,eventParam) {
    var Self = this;
    this.window = window;
    this.selected = selected;
    this.elements = [];
    this.width=width;
    this.style = style;
    this.targetDiv = null;
    this.valueIsHTML = false; //the value is a node
    this.form = false;
    this.viewValue = false;
    this.disabled = false;
    this.optWidth;
    this.show_buttons = false;
    this.special = true;
    this.keySearch = {};
    
    this.rootDiv = element;
    this.rootDiv.innerHTML = "";
    this.headTable = newnode('table',0,{
                     cellPadding: 0, cellSpacing: 0, border: 0});
                     
    this.headTable.style.tableLayout="fixed";
    this.headTableBody = document.createElement("tbody");
    this.headTableRow = document.createElement("tr");
    this.headTableColumn1 = document.createElement("td");    
    this.headTableColumn1.setAttribute("nowrap","nowrap");
    this.headTableColumn1.style.whiteSpace = "nowrap";
    this.headTableColumn1div = newnode("div",{textAlign:"center"});    
    this.headTableColumn1.appendChild(this.headTableColumn1div);
    this.headTableColumn1div.style.overflow = "hidden";    
    this.myboxComboDiv = newnode("div");
    jQuery(this.myboxComboDiv).click(function(evt){
    	// handle click on combo layer
        showHideBox3(Self,evt);
        return false;
    });
    
    if (this.style) {
        this.headTable.className="smallpopupborder smallpopupnotactive background-color-content font-color-default";
        this.headTableColumn1.className="smallpopupborderright border-color-design comboboxColumn1";
    }
    this.rootDiv.style.width=this.width;
    this.rootDiv.className="dropdownbox";
    
    this.currentValues = [];
    
    this.currentResult = "";
    
    if(eventname) {
        this.onChange = function(key, key_before) {
            triggerEvent(eventname, key, eventParam);
        };
    }
    
    if(maxscrollbar) {
        this.scrollbar=maxscrollbar;
    } else { 
        this.scrollbar=null;
    }
        
    this.setFirstDiv();

//extends.. Combobox3
    this.scrollToSelected = ComboBox3.impl.prototype.scrollToSelected;    
    this.setKey = ComboBox3.impl.prototype.setKey;        
    this.addElement = ComboBox3.impl.prototype.addElement;    
    this.sort = ComboBox3.impl.prototype.sort;
    
    // overlay
    this.overlay = createComboBoxOverlay(this);
}
ComboBoxSpecial.prototype = {
    setDynamicKey: function (key,noevent) {
        this.elements = [];
        for(var nIndx=-3;nIndx< 3;nIndx++) {
            this.addElement(noI18n(nIndx+key), nIndx+key);
        }
        this.getDomNode();        

        var myElValues = [];
        var newIndex = key;
        var bFound = false;
        for(i=0; i<this.elements.length;i++) {
            myElValues = this.elements[i];
                    if (key == myElValues.key) {
                        key = myElValues.key;
                        newIndex = i;
                        bFound = true;
                        break;
                    }               
        }
        if(!bFound) {
            return false;
        }
        if(this.elements[newIndex]) {
            changePopUpMenuText3(
                    this,
                    newIndex,
                    noevent
            );
        }                    
        return true;        
    },        
    setFirstDiv:function () {
        var Self = this;
                                
        Self.clickOnCombobox = function (e) { 
                    if (!Self.disabled) {
                        showHideBox3(Self,e);
                    }
                    stopEvent(e);
        };
        Self.headTable.style.width=this.width;    
        setTimeout(function () {
            var tmpW = Self.headTable.offsetWidth - 16;
            Self.headTableRow.appendChild(Self.headTableColumn1);    
            Self.headTableBody.appendChild(Self.headTableRow);
            Self.headTable.appendChild(Self.headTableBody);
             
            Self.window.addDOMEvent(Self.headTable,
             "click", Self.clickOnCombobox);  
             
             Self.window.addDOMEvent(Self.headTable,
             "mouseover", function() { globalclickdisable2=true; }); 
             
             Self.window.addDOMEvent(Self.headTable,
             "mouseout", function() { globalclickdisable2=false; });  
        }, 0);        
        
        Self.rootDiv.appendChild(Self.headTable);        
    },    
    scrollElements : function (sDirection){
        if(sDirection == "up") {
            var nOldVal = this.elements[0].value-6;
            this.elements = [];
            for ( var nIndx = 0; nIndx < 6; nIndx++) {
                this.addElement(noI18n(nOldVal + nIndx), nOldVal + nIndx);
            }
            this.getList();                
        }
        else {
            var nOldVal = this.elements[this.elements.length-1].value;
            this.elements = [];
            for(var nIndx=1;nIndx<= 6;nIndx++) {
                this.addElement(noI18n(nOldVal+nIndx), nOldVal+nIndx);
            }
            this.getList();                            
        }            
    },
    
    getDomNode: function () {
        this.currentValues = this.elements[this.selected];
        this.currentValue = this.currentValues.value;
        this.currentResult = this.currentValues.key;    
        while (this.headTableColumn1div.childNodes.length>0) {
            this.headTableColumn1div.removeChild(this.headTableColumn1div.firstChild);
        }
        this.currentI18nNode = new I18nNode(this.currentValues.callback);
        this.headTableColumn1div.appendChild(this.currentI18nNode.node);

        var Self = this;        
        if (this.optWidth) {
            this.myboxComboDiv.style.width = this.optWidth;
        }
        
        Self.myboxComboDiv.className="smallpopupborder smallpopuplist background-color-content font-color-default";
        Self.myboxComboDiv.style.display="none";
        Self.myboxComboDiv.style.textAlign="left";            
        
        Self.getList();
        document.body.appendChild(Self.myboxComboDiv);
        Self.myboxComboDiv.style.overflowX="hidden";
                        
    },
    getList: function () {
        var Self = this;
        if(this.scrollbar && this.scrollbar != "" && typeof this.scrollbar != "undefined") {
            if (Self.show_buttons) {
                var nfactor = 17;
            } else {
                var nfactor = 19.5;
            }
            var tempHeight = Math.floor(this.scrollbar*nfactor);

        }        
        while (this.myboxComboDiv.childNodes.length > 0) {
            this.myboxComboDiv.removeChild(this.myboxComboDiv.firstChild);
        }
        if(Self.show_buttons) {
            Self.myboxComboDiv.style.overflow = "hidden";
            Self.myboxComboDiv.style.height=(tempHeight + 22)/pxPerEm + "em";        
            var oDomUp = newnode("div",0,{className:"height10 background-color-content"});
            oDomUp.style.paddingBottom ="2px";            
            oDomUp.style.overflow="hidden";
            oDomUp.style.textAlign = "center";    
            oDomUp.style.height = "10px";            
            oDomUp.style.lineHeight = "10px";            
            oDomUp.appendChild(newnode('img',{marginBottom:"2px"},{src : getFullImgSrc("img/arrows/sort_up.gif")}));
            //select on mouse over
            Self.window.addDOMEvent(oDomUp,
                "mouseover", function() { changePopUpMenuStatus3(oDomUp, true, Self); });                        
                
            Self.window.addDOMEvent(oDomUp,
                "mouseout", function() { changePopUpMenuStatus3(oDomUp, false, Self); });             
               
            Self.rowHeight = oDomUp.offsetHeight;
            Self.window.addDOMEvent(oDomUp,"click",function (e){Self.scrollElements("up");stopEvent(e);return false;});
            Self.myboxComboDiv.appendChild(oDomUp);
            
        } else {
            Self.myboxComboDiv.style.height=tempHeight + "px";
        }
        for (var el in Self.elements) {
            (function (n) {
                var childDiv = null;
                var myValues = [];
                childDiv = newnode("div", {overflow : "hidden"});
                childDiv.style.paddingLeft ="2px";        
                childDiv.style.paddingRight ="2px";        
                childDiv.style.width="96%";
                if (Self.show_buttons) {
                    childDiv.className="height14 background-color-content";
                } else {
                    childDiv.className="height16 background-color-content";
                }
                
                Self.rowHeight = childDiv.offsetHeight;
                Self.window.addDOMEvent(childDiv,
                    "click", function() { 
                             changePopUpMenuText3(Self,n);                                                    
                         }
                   );                        
                    
                Self.window.addDOMEvent(childDiv,
                    "mouseover", function() { changePopUpMenuStatus3(childDiv, true, Self); });                        
                    
                Self.window.addDOMEvent(childDiv,
                    "mouseout", function() { changePopUpMenuStatus3(childDiv, false, Self); });      
                                         
                   myValues = Self.elements[n];
                   if(Self.viewValue) {
                        if (myValues.key != "") {
                            childDiv.className = "height16 font-color-default background-color-content";
                        } else {
                            childDiv.className = "height16 font-color-disabled background-color-content";
                        }
                   }
                myValues.i18nNode = new I18nNode(myValues.callback);
                childDiv.appendChild(myValues.i18nNode.node);
                    
                Self.myboxComboDiv.appendChild(childDiv);
                 
            })(el);
        }
        if(Self.show_buttons) {                
            var oDomDown = newnode("div",{textAlign:"center"}, {className : "background-color-content"});
            oDomDown.style.paddingBottom ="2px";        
            oDomDown.style.height = "10px";                                                                    
            oDomDown.style.lineHeight = "10px";                        
            oDomDown.appendChild(newnode('img',{marginBottom:"2px",marginTop:"2px"},{src : getFullImgSrc("img/arrows/arrow-down.gif")}));            
            //select on mouse over
            Self.window.addDOMEvent(oDomDown,
                "mouseover", function() { changePopUpMenuStatus3(oDomDown, true, Self); });                        
                
            Self.window.addDOMEvent(oDomDown,
                "mouseout", function() { changePopUpMenuStatus3(oDomDown, false, Self); });             
                   
            Self.window.addDOMEvent(oDomDown,"click",function (e){Self.scrollElements("down");stopEvent(e);return false;});        
            Self.myboxComboDiv.appendChild(oDomDown);
        }        
    }
    
};

function showHideBox3(Self, e) {
    
    if (activeComboBox) {
        
        // hide
        Self.overlay.hide();
        Self.myboxComboDiv.style.display="none";
        jQuery(activeComboBox.myboxComboDiv).hide();
        if (Self.onHide) {
            Self.onHide();
        }
        activeComboBox = null;
        
    } else {
        
        // show
        Self.overlay.show();
        
        if (jQuery(Self.rootDiv).outerWidth() > jQuery(Self.myboxComboDiv).innerWidth()) {
            jQuery(Self.myboxComboDiv).width(jQuery(Self.rootDiv).outerWidth());
        } else if (!Self.resizedWidth){
            jQuery(Self.myboxComboDiv).width(jQuery(Self.myboxComboDiv).innerWidth() + 25);
        }
        Self.resizedWidth = true;
        
        if (!document.all) {
            Self.myboxComboDiv.style.marginLeft = -1 + "px";
        }
        
        // open properly
        // box
        var box = jQuery(Self.rootDiv),
            // drop-down
            drop  = jQuery(Self.myboxComboDiv),
            // box height
            bh = box.outerHeight(),
            // box position
            pos = box.offset(),
            // height of drop-down
            h = drop.outerHeight(),
            // drop's offset parent
            p = jQuery("body"),
            // offset parent's height
            ph = p.height();
        
        // adjust position
        drop.css({
            zIndex: 40001,
            margin: "0px",
            left: (pos.left) + "px"
        });
        
        // enough space to appear below?
        if (ph > (pos.top + bh + h)) {
            drop.css("top", (pos.top + bh) + "px");
        } else {
            drop.css("top", Math.max(0, pos.top - h) + "px");
        }
        
        drop.show();
        
        // attach current active object to global object
        activeComboBox = Self;
        Self.scrollToSelected();
        
        if (Self.onShow) {
            Self.onShow();
        }
    }
}

function changePopUpMenuStatus3(element,activated, Self) {
    //if(!IE6) return;
    if(activated) {
        element.className = element.className.replace(/background-color-content/,"background-color-PMG-selection-elements");
    } else { 
        element.className = element.className.replace(/background-color-PMG-selection-elements/,"background-color-content");
    }
}

var globalclickdisable=false;

function changePopUpMenuStatus(elementid,activated) {
    var d=document.getElementById(elementid);
    if (activated) {
        d.className="background-color-PMG-selection-elements border-color-PMG-selection-elements font-color-default";
    } else {
        d.className="smallpopupnotactive background-color-content font-color-default";
    }
}
fileloaded();
/**
 *
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 *
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 *
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com
 *
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 *
 */

var menulastviews = new Object();
var lastCalendarTimeRanges = { detail: "overview" }, configparentfolder = -1;
var menudisableattachment = false, menucountunlockable = 0;
var menucounteditable = 0, menucountselected = 0;
var menucountsubselected = 0, menunestedmessage = false;
var menuisdraftfolder = false, menuselectedobjects = new Array();
var menuselectedfolders = new Object(), menuglobalzaehler=0;
var menumailfoldertreecopy = false, menucountlockable = 0;

function initContents(modules) {
    if (modules !== "core") {
        return;
    }
    register("Loaded", function () {
        menumailloaded();
        menucalendarloaded();
        menucontactloaded();
        menuinfostoreloaded();
        menuregisterMailContents();
        menuregisterCalendarContents();
        menuregisterContactContents();
        menuregisterTaskContents();
        if (configGetKey("modules.infostore.module")) {
            menuregisterInfoStoreContents();
        }
        register("OX_Configuration_Move_Folder", function () {
            // get folder first in order to get its module
            ox.api.folder.get({
                folder: configfolder,
                success: function (data) {
                    createMenuPopUpTree("menu_configuration_folder_folderpopup",
                            data.module, function (data) {
                        jQuery("#menu_configuration_folder_folderpopup").hide();
                        ox.api.folder.get({
                            folder: configfolder,
                            success: function (folder) {
                                ox.api.folder.move(folder, data);
                            }
                        });
                    }, 4); // create subfolders
                    jQuery("#menu_configuration_folder_folderpopup").show();
                }
            });
        });
    });
}

function menumailloaded() {
    // create "Mark as" submenu
    var markas_submenu = new PopupMenu();
    var entries = {
            seen: _("Read"), unseen: _("Unread"),
            answer: _("Answered"), answered: _("Unanswered"),
            "delete": _("Deleted"), undelete: _("Undeleted")
    };
    (function fillItems(menu) {
        var items = {};
        for (var i in entries) {
            var item = items[i] = new MenuItem(entries[i], (function(flag) {
                return function() { triggerEvent("OX_Mail_Flag", flag); };
            })(i));
            menu.addItem(item);
        }
        menu.onShow = function() {
            var r = menucheckRight("READ");
            items.seen.setVisible(r);
            items.unseen.setVisible(r);
            var d = menucheckRight("DELETE");
            items["delete"].setVisible(d);
            items.undelete.setVisible(d);
            var w = menucheckRight("WRITE");
            items.answer.setVisible(w);
            items.answered.setVisible(w);
        };
    })(markas_submenu);

    var menu = globalContextMenus.mail = new ContextMenu();
    function one() { this.setEnabled(this.getContext().count == 1); }
    function many() { this.setEnabled(this.getContext().count > 0); }
    var items = [
    {
        text: _("Reply"), icon: "img/mail/reply",
        event: "OX_Mail_Reply", show: one
    }, {
        text: _("Reply all"), icon: "img/mail/reply_all",
        event: "OX_Mail_ReplyAll", show: one
    }, {
        text: _("Forward"), icon: "img/mail/forward",
        event: "OX_Mail_Forward", show: many
    }, {
        text: _("Delete"), icon: "img/menu/delete",
        event: "OX_Mail_Delete", show: function() {
            this.setEnabled(this.getContext().count > 0 && menucheckRight("DELETE"));
        }
    }, {
        text: _("Print"), icon: "img/menu/print",
        event: "OX_Print", show: many
    }, {
        //-----------------------
    }, {
        text: _("Mark as"), icon: "img/mail/mark_as",
        items: markas_submenu, show: many
    }, {
        //-----------------------
    }, {
        text: _("Save"), icon: "img/menu/save_as_link",
        event: "OX_Mail_SaveMail", show: many
    }, {
        text: _("Copy"), icon: "img/menu/copy_mail",
        event: "OX_Mail_Copy", show: function() {
            this.setVisible(ox.upsell.visible("mail-copy-move"));
            this.setEnabled(this.getContext().count > 0 && menucheckRight("DELETE"));
        }
    }, {
        text: _("Move"), icon: "img/menu/mail_move",
        event: "OX_Mail_Move", show: function() {
            this.setVisible(ox.upsell.visible("mail-copy-move"));
            this.setEnabled(this.getContext().count > 0);
        }
    }, {
        //-----------------------
    }, {
        text: _("Edit draft"), icon: "img/menu/edit_draft",
        icon_d: "img/dummy.gif", event: "OX_Mail_Edit_Draft",
        show: function() {
            this.setVisible(menuisdraftfolder);
            this.setEnabled(this.getContext().count == 1);
        }
    }, {
        text: _("Show source"), icon: "img/menu/view_emailsource",
        event: "OX_Mail_Show_Source", show: one
    }, {
        text: _("Spam"), icon: "img/mail/spam",
        event: "OX_Mail_Spam", show: function() {
            this.setVisible(   this.getContext().count > 0
                            && configGetKey("mail.spambutton")
                            && checkSpamFolder(true)
                            && getMailAccountIdByFolder(this.getContext().getSelected()[0].folder_id) == 0);
        }
    }, {
        text: _("Not Spam"), icon: "img/menu/ham",
        icon_d: "img/menu/ham.gif", event: "OX_Mail_Ham",
        show: function() {
            this.setVisible(   menucountselected > 0
                            && configGetKey("mail.spambutton")
                            && checkSpamFolder(false)
                            && getMailAccountIdByFolder(this.getContext().getSelected()[0].folder_id) == 0);
        }
    }];
    for (var i in items) {
        (function(def) {
            if (def.text) {
                var item = new MenuItem(def.text,
                        def.event ? function() { triggerEvent(def.event); } :
                            def.items
                );
                item.setIcon(def.icon + ".gif", def.icon_d || def.icon + "_d.gif");
                item.onShow = def.show;
            } else {
                var item = new MenuSeparator();
            }
            menu.addItem(item);
        })(items[i]);
    }
}
function menuregisterMailContents() {
    register("OX_Mail_Move",menumailmoveclicked);
    register("OX_Mail_Copy",menumailcopyclicked);
}

function menumailmoveclicked() {
    menumailfoldertreecopy=false;
    createMenuPopUpTree("menu_mail_move_foldertree","mail", doMailsFolder, 2); // create objects
    document.getElementById("menu_mail_move_foldertree").style.display="";
}

function menumailcopyclicked() {
    menumailfoldertreecopy=true;
    createMenuPopUpTree("menu_mail_move_foldertree","mail", doMailsFolder, 2); // create objects
    document.getElementById("menu_mail_move_foldertree").style.display="";
}

function doMailsFolder(data) {
    var selectedfolderid = data.id;
    if(menumailfoldertreecopy) {
        triggerEvent("OX_Object_Copy",selectedfolderid);
    } else {
        triggerEvent("OX_Object_Move",selectedfolderid);
    }
    document.getElementById("menu_mail_move_foldertree").style.display="none";
}

/**
 * Checks if all selected objects are outside/inside the spam folder.
 * Used to decide whether the Spam/Ham button should be displayed.
 * @param {Boolean} type Whether to check if all mails are outside (true, used
 * for the Spam button) or inside (false, used for the Ham button) the spam
 * folder.
 * @type Boolean
 * @return true if all selected objects are outside/inside the spam folder.
 * false if at least one selected object isn't, and therefore couldn't be moved
 * by the Spam/Ham button.
 */
function checkSpamFolder(type) {
    var spam = configGetKey("mail.folder.spam");
    for(var i in menuselectedobjects) {
        var folder = menuselectedobjects[i].folder;
        if(!folder) folder= menuselectedobjects[i].folder_id;
        if ((folder == spam) == type) return false;
    }
    return true;
}

/****************************************************************************************************/
/*** Calendar Function ******************************************************************************/
/****************************************************************************************************/
function menucalendarloaded() {
    var contextData = [];
    var count = Value(), writable = Value(), deletable = Value(), isPrivate = Value();
    var confirmable = Value(), acceptDenyVisible = Value();

    function drawCatsList(cats_sub, cats) {
        // remove all sub nodes first
        cats_sub.removeAllItems();

        // delete menu
        var del = new MenuItem(_("Delete all Categories"),
                function() { triggerEvent('OX_Delete_All_Categories'); }
        );
        del.setEnabledF(function() { return cats.length > 0; });
        cats_sub.addItem(del);

        // spacer
        cats_sub.addItem(new MenuSeparator());

        // category menu items (with color div)
        var categories = (ox.categories.list() || []);
        var $l = (categories.length > 10 ? 10 : categories.length);
        for (var i=0; i < $l; i++) {
            var item = new MenuItem(noI18n(categories[i].name),
                (function(category) {
                    return function() {
                        triggerEvent('OX_Add_Categories', [ category ]);
                    };
                })(categories[i].name));
            cats_sub.addItem(item);

            // drawing color divs and append as new menu item
            var node = item.node;
            var text = node.firstChild;
            removeChildNodes(node);
            node.appendChild(newnode("div",
                    { flt: "left", margin: "3px 5px 0 0" },
                    { className: "colorLabelCont bs ha colorLabel" + (categories[i].color || 0) }));
            node.appendChild(text);
        }

        // spacer
        cats_sub.addItem(new MenuSeparator());

        // all categories menu item (to open dialog)
        cats_sub.addItem(
                new MenuItem(_("All Categories..."),
                    function() { triggerEvent('OX_All_Categories'); }
                )
        );
    }

    var menu = globalContextMenus.calendar = new ContextMenu();

    var edit = new MenuItem(_("Edit"),
        function() { triggerEvent('OX_Calendar_Edit'); });
    edit.setIcon("img/menu/edit.gif", "img/menu/edit_d.gif");
    edit.setEnabledF(function() {
        return count() == 1 && writable() && !isPrivate();
    });
    menu.addItem(edit);

    var cats_s1 = new PopupMenu();
    cats_s1.onShow = function() {
        // getting selected apps
        var apps = contextData;
        // create consolidated list of categories
        var cats = [];
        for (var i in apps) {
            if (apps[i].categories) {
                cats.push(apps[i].categories);
            }
        }
        drawCatsList(cats_s1, cats);
    };

    var cats = new MenuItem(_("Categories"),
            cats_s1);
    cats.setIcon("img/mail/mark_as.gif", "img/mail/mark_as_d.gif");
    cats.setEnabledF(function() { return count() > 0 && !isPrivate(); });
    menu.addItem(cats);

    var del = new MenuItem(_("Delete"),
        function() { triggerEvent('OX_Calendar_Delete'); });
    del.setIcon("img/menu/delete.gif", "img/menu/delete_d.gif");
    del.setEnabledF(function() { return count() > 0 && deletable() && !isPrivate(); });
    menu.addItem(del);

    menu.addItem(new MenuSeparator());

    var confirm = new MenuItem(_("Change confirmation"), function() {
        var objects = new Array(contextData.length);
        for (var i = 0; i < objects.length; i++) {
            objects[i] = new AcceptDenyObject("calendar", contextData[i]);
        }
        (new AcceptDeny(true)).showConfirmation(objects);
    });
    confirm.setIcon("img/menu/global_confirmation_change.gif",
                    "img/menu/global_conf_change_d.gif");
    confirm.setEnabledF(function() {
        return count() > 0 && !acceptDenyVisible() && confirmable() && !isPrivate();
    });
    menu.addItem(confirm);

    menu.onShow = function() {
        acceptDenyVisible.set(AcceptDeny.visible);
        var objects = clone(this.getContext().getSelected());
        count.set(objects.length);
        if (!objects.length) return;
        for (var i = 0; i < objects.length; i++) {
            objects[i].module = "calendar";
            objects[i].folder_id = objects[i].folder;
            delete objects[i].folder;
        }
        writable.clear();
        deletable.clear();
        confirmable.clear();
        isPrivate.clear();
        var collection = { columns: [ "created_by", "users", "title", "location",
                                     "start_date", "end_date", "categories", "private_flag" ],
                           objects: objects };

        OXCache.newRequest(null, "calendar", collection, null, responseData);

        function responseData(data) {
            contextData = data.objects;
            ox.api.folder.get({
                folder: activefolder,
                success: function(response){
                    handleResponse(response, contextData);
                }
            });
        }
        function handleResponse(folder, contextData) {
            writable.set(menuhasRight(contextData, "WRITE"));
            deletable.set(menuhasRight(contextData, "DELETE"));
            if (!writable.value) {
                confirmable.set(false);
            }
            isPrivate.set(false);
            var shared = folder.created_by != configGetKey('identifier');
            var tmpPrivate = false;
            var me = folder.type == 3 ? folder.created_by
                                      : configGetKey("identifier");
            for (var i in contextData) {
                var users = contextData[i].users;
                if (shared == true && contextData[i].private_flag == true) {
                    isPrivate.set(true);
                }
                CheckUsers: {
                    for(var j = 0; j < users.length; j++) {
                        if (users[j].id == me) {
                            confirmable.set(true);
                            break CheckUsers;
                        }
                    }
                    confirmable.set(false);
                    return;
                }
            }
        }
    };


    // Team view context menu
    var menu = globalContextMenus.teamview = new ContextMenu();

    // Edit
    var edit = new MenuItem(_("Edit"),
        function() { triggerEvent('OX_Calendar_Edit'); });
    edit.setIcon("img/menu/edit.gif", "img/menu/edit_d.gif");
    edit.setEnabledF(function() { return count() == 1; });
    menu.addItem(edit);

    var cats_s2 = new PopupMenu();
    cats_s2.onShow = function() {
        // getting selected apps
        var apps = clone(this.getContext().getSelectedItems());
        // create consolidated list of categories
        var cats = [];
        for (var i in apps) {
            if (apps[i].categories) {
                cats.push(apps[i].categories);
            }
        }
        drawCatsList(cats_s2, cats);
    };

    var cats = new MenuItem(_("Categories"),
            cats_s2);
    cats.setIcon("img/mail/mark_as.gif", "img/mail/mark_as_d.gif");
    cats.setEnabledF(function() { return count() > 0; });
    menu.addItem(cats);

    // Delete
    var del = new MenuItem(_("Delete"),
        function() { triggerEvent('OX_Calendar_Delete'); });
    del.setIcon("img/menu/delete.gif", "img/menu/delete_d.gif");
    del.setEnabledF(function() { return count() > 0; });
    menu.addItem(del);

    menu.onShow = function() {
        // get number of selected appointments
        var c = this.getContext().numSelected();
        count.set(c);
    };
}

function menuregisterCalendarContents() {
    register("OX_Calendar_Move", menucalendarmoveclicked);
}
function menucalendarmoveclicked() {
    createMenuPopUpTree("menu_calendar_move_foldertree", "calendar",
        moveCalendarFolder, 2); // create objects
    jQuery("#menu_calendar_move_foldertree").show();
}
function moveCalendarFolder(data) {
    var selectedfolderid = data.id;
    triggerEvent("OX_Object_Move", selectedfolderid);
    jQuery("#menu_calendar_move_foldertree").hide();
}

/****************************************************************************************************/
/*** Task Function ******************************************************************************/
/****************************************************************************************************/
function menutasksloaded() {
    addOnClose($('menu_tasks_move_foldertree'), function () {
        $('menu_tasks_move_foldertree').style.display = "none";
    });
}

function menuregisterTaskContents() {
    register("OX_Task_Move", menutaskmoveclicked);
}

function menutaskmoveclicked() {
    createMenuPopUpTree("menu_tasks_move_foldertree","tasks",
        moveTasksFolder, 2); // create objects
    document.getElementById("menu_tasks_move_foldertree").style.display="";
}

function moveTasksFolder(data) {
    var selectedfolderid = data.id;
    triggerEvent("OX_Object_Move",selectedfolderid);
    document.getElementById("menu_tasks_move_foldertree").style.display="none";
}

/****************************************************************************************************/
/*** Contact Function ******************************************************************************/
/****************************************************************************************************/
var menucontactfoldertreecopy=false;
function menucontactloaded() {
    var menu = globalContextMenus.contacts = new ContextMenu();
    var count = Value(), writable = Value(), deletable = Value();

    // Edit contact
    var edit = new MenuItem(_("Edit"),
        function() { triggerEvent('OX_Contact_Edit'); });
    edit.setIcon("img/menu/edit.gif", "img/menu/edit_d.gif");
    edit.setEnabledF(function() { return count() == 1 && writable(); });
    menu.addItem(edit);

    // Delete contact
    var del = new MenuItem(_("Delete"),
        function() { triggerEvent('OX_Contact_Delete'); });
    del.setIcon("img/menu/delete.gif", "img/menu/delete_d.gif");
    del.setEnabledF(function() { return count() > 0 && deletable(); });
    menu.addItem(del);

    menu.addItem(new MenuSeparator());

    // Copy contact
    var copy = new MenuItem(_("Copy"),
        function() { triggerEvent('OX_Contact_Copy'); });
    copy.setIcon("img/menu/contact_copy.gif", "img/menu/contact_copy_d.gif");
    copy.setEnabledF(function() { return count() > 0 && ox.upsell.visible("contact-copy-move"); });
    menu.addItem(copy);

    // Move contact
    var move = new MenuItem(_("Move"),
        function() { triggerEvent('OX_Contact_Move'); });
    move.setIcon("img/menu/contact_move.gif", "img/menu/contact_move_d.gif");
    move.setEnabledF(function() { return count() > 0 && deletable() && ox.upsell.visible("contact-copy-move"); });
    menu.addItem(move);

    // Use contact as template
    var template = new MenuItem(_("Use as template"),
        function() { triggerEvent('OX_Contact_Duplicate'); });
    template.setIcon("img/menu/duplicate_contacts.gif",
                     "img/menu/duplicate_contacts_d.gif");
    template.setEnabledF(function() { return count() == 1 && writable(); });
    menu.addItem(template);

    menu.addItem(new MenuSeparator());

    // Send email to contact
    var sendMailTo = new MenuItem(_("Send E-Mail"),
        function() { triggerEvent('OX_Contact_SendMail'); });
    sendMailTo.setIcon("img/mail/write_mail.gif", "img/mail/write_mail.gif");
    sendMailTo.setEnabledF(function() { return count() > 0; });
    menu.addItem(sendMailTo);

    // Send contact as vCard
    var sendVCard = new MenuItem(_("Send as vCard"),
        function() { triggerEvent('OX_Contact_SendVCard'); });
    sendVCard.setIcon("img/mail/forward.gif", "img/mail/forward_d.gif");
    sendVCard.setEnabledF(function() { return count() > 0; });
    menu.addItem(sendVCard);

    // context menu - onShow handler
    menu.onShow = function() {
        var collection = { columns: ["created_by"],
                           objects: this.getContext().getSelected() };
        count.set(collection.objects.length);
        if (!count.value) return;
        OXCache.newRequest(null, "contacts", collection, null, responseData);
        function responseData(data) {
            writable.set(menuhasRight(data.objects, "WRITE"));
            deletable.set(menuhasRight(data.objects, "DELETE"));
        }
    };
}

function menuregisterContactContents() {
    register("OX_Contact_Move",menucontactmoveclicked);
    register("OX_Contact_Copy",menucontactcopyclicked);
}
function menucontactmoveclicked() {
    createMenuPopUpTree("menu_contacts_move_foldertree", "contacts",
        doContactsFolder, 2); // create objects
    document.getElementById("menu_contacts_move_foldertree").style.display="";
    menucontactfoldertreecopy=false;
}

function menucontactcopyclicked() {
    createMenuPopUpTree("menu_contacts_move_foldertree", "contacts",
        doContactsFolder, 2); // create objects
    document.getElementById("menu_contacts_move_foldertree").style.display="";
    menucontactfoldertreecopy=true;
}

function doContactsFolder(data) {
    var selectedfolderid = data.id;
    if (menucontactfoldertreecopy) {
        triggerEvent("OX_Object_Copy",selectedfolderid);
    } else {
        triggerEvent("OX_Object_Move",selectedfolderid);
    }
    document.getElementById("menu_contacts_move_foldertree").style.display="none";
}

/****************************************************************************************************/
/*** InfoStore Function ******************************************************************************/
/****************************************************************************************************/

function menuinfostoreloaded() {
    var menu = globalContextMenus.infostore = new ContextMenu();
    var contextData = [];
    var count = Value(), readable = Value(), writable = Value();
    var editable = Value(), deletable = Value(), lockable = Value();
    var unlockable = Value(), hasFile = Value();;

    var edit = new MenuItem(_("Edit"),
        function() { triggerEvent('OX_InfoStore_Edit'); });
    edit.setIcon("img/menu/edit.gif", "img/menu/edit_d.gif");
    edit.setEnabledF( function() {
        return count() == 1 && writable() && editable();
    });
    menu.addItem(edit);

    var del = new MenuItem(_("Delete"),
        function() { triggerEvent('OX_InfoStore_Delete'); });
    del.setIcon("img/menu/delete.gif", "img/menu/delete_d.gif");
    del.setEnabledF(function() {
        return count() > 0 && (lockable() || unlockable()) && deletable();
    });
    menu.addItem(del);

    menu.addItem(new MenuSeparator());

    var lock = new MenuItem(_("Lock"),
        function() { triggerEvent('OX_InfoStore_Lock'); });
    lock.setIcon("img/menu/sperren.gif", "img/menu/sperren_d.gif");
    lock.setEnabledF(function() {
        return count() > 0 && writable() && lockable();
    });
    menu.addItem(lock);

    var unlock = new MenuItem(_("Unlock"),
        function() { triggerEvent('OX_InfoStore_Unlock'); });
    unlock.setIcon("img/menu/entsperren.gif", "img/menu/entsperren_d.gif");
    unlock.setEnabledF(function() {
        return count() > 0 && writable() && unlockable();
    });
    menu.addItem(unlock);

    menu.addItem(new MenuSeparator());

    var attach = new MenuItem(_("Send as attachment"),
        function() { triggerEvent('OX_InfoStore_Send_Attachment'); });
    attach.setIcon("img/menu/alsanlagesenden.gif",
                   "img/menu/alsanlagesenden_d.gif");
    attach.setEnabledF(function() {
        return configGetKey("modules.mail.module") &&
            count() > 0 && readable() && hasFile();
    });
    menu.addItem(attach);

    var link = new MenuItem(_("Send as link"),
        function() { triggerEvent('OX_InfoStore_Send_Link'); });
    link.setIcon("img/menu/save_as_link.gif", "img/menu/save_as_link_d.gif");
    link.setEnabledF(function() {
        return ("infostore_sendLink" in window) && count() > 0 && readable();
    });
    menu.addItem(link);

    var open = new MenuItem(_("Open"), function() {
        triggerEvent('OX_InfoStore_Open_Attachment', contextData);
    });
    open.setIcon("img/mail/open_ISattachment.gif",
                 "img/mail/open_ISattachment_d.gif");
    open.setEnabledF(function() { return count() >= 1 && hasFile(); });
    menu.addItem(open);

    var save = new MenuItem(_("Save"), function() {
        triggerEvent("OX_InfoStore_Save_Attachment", contextData);
    });
    save.setIcon("img/menu/attachment_save.gif",
                 "img/menu/attachment_save_d.gif");
    save.setEnabledF(function() { return count() >= 1 && hasFile(); });
    menu.addItem(save);

    menu.onShow = function() {
        var collection = {
            columns: ["created_by", "modified_by", "locked_until", "file_size",
                      "filename"],
            objects: this.getContext().getSelected()
        };
        count.set(collection.objects.length);
        if (!count.value) return;
        readable.clear();
        writable.clear();
        editable.clear();
        deletable.clear();
        lockable.clear();
        unlockable.clear();
        hasFile.clear();
        OXCache.newRequest(null, "infostore", collection, null, responseData);
        function responseData(data) {
            contextData = data.objects;
            var l = true, ul = true, e = true;
            var me = configGetKey("identifier");
            for (var i in contextData) {
                var item = contextData[i];
                if (!item.locked_until) ul = false; else {
                    l = false;
                    if (item.modified_by != me) {
                        e = false;
                        if (item.created_by != me) ul = false;
                    }
                }
            }
            lockable.set(l);
            unlockable.set(ul);
            editable.set(e);
            menuhasRightandLoad(contextData, "READ",
                function(can) {
                    readable.set(can);
                    hasFile.set(can && hasAttachment(contextData));
                });
            menuhasRightandLoad(contextData, "WRITE",
                function(can) { writable.set(can); });
            menuhasRightandLoad(contextData, "DELETE",
                function(can) { deletable.set(can); });
        }
    };

    function hasAttachment(myobjects) {
        for(var i in myobjects) {
            if(!(myobjects[i]["file_size"]>0)) {
                return false;
            }
        }
        return true;
    };
}
function menuregisterInfoStoreContents() {
    register("OX_InfoStore_Move",menuinfostoremoveclicked);
    register("OX_Search_InfoStore",onmenusearchInfoStore);
}

function onmenusearchInfoStore() {
    var mysearchparam=document.getElementById("menu_search_infostore_searchfield").value;
    var mysearch=new Object();
    mysearch["pattern"]=mysearchparam;
    triggerEvent("OX_InfoStore_Search",mysearch);
}

function menuinfostoremoveclicked() {
    register("OX_InfoStore_Move",menuinfostoremoveclicked);
    createMenuPopUpTree("menu_infostore_move_foldertree", "infostore",
        moveInfoStoreFolder, 2); // create objects
    document.getElementById("menu_infostore_move_foldertree").style.display="";
}

function moveInfoStoreFolder(data) {
    var selectedfolderid = data.id;
    triggerEvent("OX_Object_Move",selectedfolderid);
    document.getElementById("menu_infostore_move_foldertree").style.display="none";
}

function menucheckRight(right) {
    return menuhasRight(menuselectedobjects,right);
}
function menuhasRightandLoad(selectedobjects,right,cbtrue,cbfalse) {
    var folders = {};
    for(var i in selectedobjects) {
        var folder = selectedobjects[i];
        var id = folder.folder || folder.folder_id;
        folders[id] = true;
    }
    var folderList = [];
    for (var i in folders) folderList.push(i);
    ox.api.folder.getMultiple({
        list: folderList,
        success: function (data) {
            if (menuhasRight(selectedobjects, right, data)) {
                if (cbtrue) cbtrue(true);
            } else {
                if (cbfalse) cbfalse(); else if (cbtrue) cbtrue(false);
            }
        }
    });
}

function menuhasRight(selectedobjects,right,resp) {
    right = { READ: 7, WRITE: 14, DELETE: 21, ADMIN: 28, USER: 29 }[right];
    if (!right) return false;
    function getFolder(folder) {
        return menuselectedfolders[folder] || resp && resp[folder];
    }
    var me = configGetKey("identifier");
    var rights = {};
    for (var i in selectedobjects) {
        var o = selectedobjects[i];
        var fid = o.folder || o.folder_id;
        if (!rights[fid]) {
            var folder = getFolder(fid);
            if (!folder) return false;
            switch (rights[fid] = computePerm(folder.own_rights, right)) {
                case 0:
                    return false;
                case 1:
                    if (right >= 28) rights[fid] = 2;
            }
        }
        if (rights[fid] == 1 && o.created_by != me) return false;
    }
    return true;
}

function createMenuPopUpTree(elementid, moduletype, executeFunction, minPerms) {
    ox.api.ui.selectFolder({
        type: moduletype,
        id: "select-folder-popup-" + moduletype,
        callback: executeFunction,
        node: $(elementid+"_content"),
        minPerms: minPerms
    });
}

function newServerError(args, level) {
	if (args.error) {
		var sizes = {
			"MSG-0028": [0, 2], "MSG-0029": [0], "NON-0005": [0, 1],
			"AJP-0006": [0], "FLS-0003": [0, 1, 2], "CON-0101": [1, 2],
			"AJP-0020": [0, 1]
		}[args.code];
		if (sizes) {
			for (var i = 0; i < sizes.length; i++) {
				args.error_params[sizes[i]] = bytesToString(args.error_params[sizes[i]]);
			}
		}
        triggerEvent("OX_New_Error", (level || 4), formatError(args), args);
	}
}

function loadDefaultModules() {
    var modules = ox.api.ui.listModules();
    for (var i in modules) {
        var configPath = "gui." + modules[i].name + ".view";
        if(!modules[i].disable && configContainsKey(configPath)) {
            var defaults = configGetKey(configPath);
            if (modules[i].name == "calendar") {
                defaults = configGetKey("gui.calendar.views");
                if (!configGetKey("modules.calendar.calendar_teamview") &&
                        defaults.view == "team") {
                    defaults.view = "calendar";
                    configSetKey("gui.calendar.views.view", "calendar");
                }
                menulastviews.calendar = getDefaultCalendarView();
                lastCalendarTimeRanges.calendar = defaults.calendar;
                lastCalendarTimeRanges.team = defaults.team;
                lastCalendarTimeRanges.list = defaults.list;
            } else {
                menulastviews[modules[i].name] = configGetKey(configPath);
            }
        } else {
            if (modules[i].name== "configuration") {
                menulastviews[modules[i].name] = "configuration/settings";
            } else {
                menulastviews[modules[i].name] = modules[i].name;
            }
        }
    }
}

function switchtoView(newview, param) {

    var elements = newview.split("/");
    var newmodule= elements[0];

    // continuation
    var cont = function () {
        if (isDefaultableView(newview)) {
            menulastviews[newmodule] = newview;
            if (newmodule === "calendar" && elements[1] !== "detail") {
                lastCalendarTimeRanges[elements[1]] = elements[2];
            }
        }
        changeView(newview, param, true);
        triggerEvent("OX_Switched");
        // toolbar resize fix
        // fixes: jump into folder props from portal (i.e. collapsed toolbar; but still in portal somehow)
        if (ox.ToolBarController && ox.ToolBarController.initialResize) {
            ox.ToolBarController.initialResize();
        }
    };

    // change module? (check for UIController since we might be in nested windows)
    // special case: configuration/folder|import does not change to configuration
    var isHybridConfig = /^configuration\/(folder|import)/.test(newview);
    if (ox.UIController && !isHybridConfig && newmodule !== ox.UIController.getModule()) {
        // set module
        ox.UIController.setModule({
            module: newmodule,
            success: cont
        });
    } else {
        cont();
    }
}

function menufollowLink(mymodule,object) {
    if (mymodule == "task") {
        mymodule = "tasks";
    }

    var view = mymodule + "/detail/overview";
    activefolder = object.folder;
    if (!object.folder_id && object.folder) {
        object.folder_id = object.folder;
    }
    if (!object.module) {
        object.module = mymodule;
    }
    menucountselected = 1;
    menucountsubselected = 0;
    var oArray = new Array();
    oArray.push(object);
    switch(mymodule) {
        case "infostore":
            view = menulastviews[mymodule] || mymodule+"/split/overview";
            if (object.id) {
                view = mymodule+"/split/overview";
                object["directLink"] = true;
                oLastSelectedInfoObject = new Array();
                oLastSelectedInfoObject[0] = object;
            }
            break;
        case "contacts":
            if (object.id) {
                object["directLink"] = true;
                contactDetailId = object;
            } else {
                view = menulastviews[mymodule] || configGetKey("gui.contacts.view");
            }
            break;
        case "calendar":
            if (object.id) {
                object["directLink"] = true;
                selectedAppointment = object;
            } else {
                view = getLastCalendarView();
            }
            break;
        case "tasks":
            view = menulastviews[mymodule] || configGetKey("gui.tasks.view");
            if (object.id) {
                view = mymodule+"/split/overview";
                object["directLink"] = true;
                activeTaskId = new Array();
                activeTaskId[0]=object;
            }
            break;
        case "mail":
            view = menulastviews[mymodule] || configGetKey("gui.mail.view");
            if (object.id) {
                if (view.match(/mail\/list/gi)) view = mymodule + "/hsplit/unthreaded";
                object["directLink"]=true;
                mail_lastSelectedId=object;
            }
    }
    var folderChanged = (activefolder == object.folder);
    // use new UI controller
    ox.UIController.setFolder({
        folder: object.folder,
        module: "default",
        view: view,
        force: "folder",
        success: function () {
            // selection only possible with id
            if (object.id) {
                triggerEvent("Selected", oArray);
            }
        }
    });
}

function folderUsersUpsell() {
    if (configGetKey("modules.folder.read_create_shared_folders")) return;
    ox.api.folder.get({
        folder: configparentfolder != -1 ? configparentfolder : configfolder,
        success: function (folder) {
            var haveadminrights = configparentfolder == -1
                ? folder.own_rights & (1 << 28) // admin flag
                : folder.own_rights & 127 >= 4; // folder permissions >= create subfolders
            var haveadminrights2 = folder.type == 5 &&
                configGetKey("modules.folder.public_folders");
            if (haveadminrights && !haveadminrights2) {
                triggerEvent("Feature_Not_Available", "modules/folders/users", window);
            }
        }
    });
}

function menuswitchConfigurationFolderView(param) {
    var newpath = currentpath2.slice(0);
    newpath[2] = param;
    newpath.length = 3;
    switchtoView(newpath.join("/"));
}

/**
 * Shows or hides a menu element in the specified view.
 * @param {String} name The name of the view as used for registerView.
 * @param {String} id The ID of the DOM node which will be shown in the view.
 * @param {Boolean} hide If true, a previously shown element will be hidden.
 */
function changeDisplay(name, id, hide) {
    var change = changeDisplay.changes[name];
    if (change) change = change[id];
    if (hide) {
        if (change) {
            delete changeDisplay.changes[name][id];
        } else {
            console.error("Invalid changeDisplay", name, id);
        }
    } else {
        if (change) {
        } else {
            var views = ox.widgets.toolBar.views;
            if (name in views) {
                MenuNodes.contexts[id].node.addTo(views[name].children[0]);
            } else {
                var changes = changeDisplay.changes[name];
                if (!changes) changes = changeDisplay.changes[name] = {};
                changes[id] = true;
            }
        }
    }
}
changeDisplay.changes = {};
changeDisplay.update = function(name) {
    if (name in changeDisplay.changes) {
        var tab = ox.widgets.toolBar.views[name].children[0];
        for (var id in changeDisplay.changes[name]) {
            MenuNodes.contexts[id].node.addTo(tab);
        }
        delete changeDisplay.changes[name];
    }
};

function menuswitchMailView(param) {
    var newpath = currentpath2.slice(0);
    newpath[1] = param;
    switch (param)  {
        case "list":
        case "vsplit":
        case "hsplit":
            if (newpath[2]) {
                if (newpath[2] != "unthreaded" && newpath[2] != "threaded") {
                    newpath.length = 3;
                    newpath[2] = "unthreaded";
                    break;
                }
            } else {
                newpath[2] = "unthreaded";
            }
            break;
    }
    switchtoView(newpath.join("/"));
}

function menuswitchCalendarView(param) {
    var range = lastCalendarTimeRanges[param];
    if (   param != "detail" && currentpath2[1] != "detail"
        && currentpath2[2] != "search"
        && configGetKey("gui.calendar.views.shared"))
    {
        range = currentpath2[2];
    }
    switchtoView("calendar/" + param + "/" + range);
}
function menuswitchCalendarTimerange(param) {
    var newpath = currentpath2.slice(0);
    newpath[2] = param;
    if (newpath[1] == "detail") {
        newpath[1] = "calendar";
        if (menulastviews["calendar"] !== undefined) {
            newpath[1] = menulastviews["calendar"].split("/")[1];
        }
    }
    newpath.length = 3;
    switchtoView(newpath.join("/"));
}

function menuswitchCalendarDetail(param) {
    var newpath = currentpath2.slice(0);
    newpath[2] = param;
    newpath.length = 3;
    switchtoView(newpath.join("/"));
}

function menuswitchContactView(param) {
    var newpath = currentpath2.slice(0);
    newpath[1] = param;
    switch (param)  {
        case "phonelist":
        case "cards":
            newpath.length = 2;
            break;
        case "detail":
            newpath[2] = "overview";
            newpath.length = 3;
            break;
    }
    switchtoView(newpath.join("/"));
}

function menuswitchContactDetail(param) {
    var newpath = currentpath2.slice(0);
    newpath[2] = param;
    newpath.length = 3;
    switchtoView(newpath.join("/"));
}

function menuswitchContactNew(param) {
    var newpath = currentpath2.slice(0);
    newpath[1] = param;
    newpath.length = 2;
    switchtoView(newpath.join("/"));
}

function menuswitchInfoStoreDetail(param) {
    var newpath = currentpath2.slice(0);
    newpath[2] = param;
    newpath.length = 3;
    switchtoView(newpath.join("/"));
}

function menuswitchTaskView(param) {
    var newpath = currentpath2.slice(0);
    newpath[1] = param;
    switch (param)  {
    case "list":
        case "split":
            newpath.length = 2;
            break;
        case "detail":
            newpath[2] = "overview";
            newpath.length = 3;
            break;
    }
    switchtoView(newpath.join("/"));
}
function menuswitchTaskDetail(param) {
    var newpath = currentpath2.slice(0);
    newpath[2] = param;
    newpath.length = 3;
    switchtoView(newpath.join("/"));
}

function menuswitchTaskNew(param) {
    var newpath = currentpath2.slice(0);
    newpath[1] = param;
    newpath.length = 2;
    switchtoView(newpath.join("/"));
}

register("OX_Configuration_Set_Empty", loadDefaultModules);
register("OX_Switch_View", switchtoView);
register("OX_Direct_Linking", function(modulename, object) {
    menufollowLink(modulename,object);
});

register("OX_Selected_Nested",function (nestedbool) {
    menunestedmessage = nestedbool;
});

register("OX_Selected_Draft",function (draftbool) {
    menuisdraftfolder = draftbool;
});

register("Selected",function (selected) {
    if(!equals(menuselectedobjects,selected)) {
        menucountselected = selected.length;
        menuselectedobjects = selected;
        var ids = {};
        for (var i = 0; i < menuselectedobjects.length; i++) {
            var object = menuselectedobjects[i];
            ids[object.folder || object.folder_id] = true;
        }
        var idList = [];
        for (var i in ids) idList.push(i);
        ox.api.folder.getMultiple({
            list: idList,
            success: function (folders) {
                menuselectedfolders = folders;
                triggerEvent("OX_SELECTED_ITEMS_CHANGED", menucountselected);
                triggerEvent("OX_SELECTION_CHANGED", menucountselected);
            }
        });
    }
});

register("SubSelected", function(selected,disable) {
    menucountsubselected = selected;
    if (disable) {
        menudisableattachment = true;
    } else {
        menudisableattachment = false;
    }
});

register("OX_Selected_Member", function(selected) {
    triggerEvent("SubSelected", selected.length);
});

register("OX_Lockable",function(lock, unlock, editlock) {
    menucountlockable = lock;
    menucountunlockable = unlock;
    menucounteditable = editlock;
});

function resizeToolbar() {
    try {
        if (ox.ToolBarController) {
            setTimeout(function() {
                ox.ToolBarController.initialResize();
            }, 0);
        }
   } catch (e) { }
}

register("OX_menu_Change_Height", resizeToolbar);

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Andreas Mayer <andreas.mayer@open-xchange.com>
 * 
 */

/*global ox, $, config, internalCache, configGetKey, newnode, _, addDOMEvent,
addTranslated, format, removeDOMEvent, removeChildNodes, jQuery, getFullImgSrc,
ComboBox3, noI18n, register, showNode, centerPopupWindow, unregister,
hideNode, OXCache, OXMailMapping, fileloaded */

/*jslint devel: true, browser: true, forin: true, unparam: true, maxerr: 50, indent: 4 */

function computePerm(nBits, nOffset) {
    return (nBits >> nOffset) & (nOffset >= 28 ? 1 : 127);
}

function FolderPath(module) {
    this.module = module;
    this.folderId = null;
    this.nodeId = null;
    this.className = "linkstyle-headline";
    this.classNameD = "font-style-low";
    this.events = [];
    this.cb = function (module, folderId) {
        return function () {
            ox.api.ui.setModule(module, null, folderId);
        };
    };
    this._cancelDraw = false;
}

FolderPath.prototype = {
    
    setFolderId: function (folderId) {
        this.folderId = folderId;
    },
    
    drawDOMNode: function (nodeId, folderId) {
        
        var Self = this;
        
        if (!this.modify_cb) {
            this.modify_cb = function (e) {
                // not mail? not local update? (mail handles this differently)
                if (e.why !== "folder.update.local" && ox.UIController.getModule() !== "mail") {
                    Self.drawDOMNode();
                }
            };
            ox.api.folder.dispatcher.bind("modify", this.modify_cb);
        }
        if (nodeId) {
            this.nodeId = nodeId;
        }
        if (folderId) {
            this.folderId = folderId;
        }
        if (!$(this.nodeId) || !this.folderId || this._cancelDraw) {
            return;
        }
        this._cancelDraw = true;
        
        // get "myself" from user interface first (it is part of most paths and usually cached)
        var userId = config.identifier;
        internalCache.getUsers([userId], function (user) {
            
            // get display name
            var myName = String(user[userId].display_name);
            
            // get path
            ox.api.folder.getParents({
                folder: Self.folderId,
                success: function (list) {
                    
                    Self.clear();
                    
                    var $l = list.length, i = $l - 1, folder;
                    for (; i >= 0; i--) {
                        
                        folder = ox.util.clone(list[i]);
                        
                        if (folder.id === "1") {
                            // overwrite title
                            folder.title = myName;
                        }
                        var read = computePerm(folder.own_rights, 7) > 0;
                        var myNode = newnode("span", 0, { className: read && i > 0  ? Self.className : Self.classNameD },
                            [ document.createTextNode(folder.title) ]);
                        if (read && i > 0) {
                            var fcb = (Self.cb)(folder.module, folder.id);
                            addDOMEvent(myNode, "click", fcb);
                            Self.events.push(["click", fcb]);
                        } 
                        $(Self.nodeId).appendChild(myNode);
                        if (i > 0) {
                            $(Self.nodeId).appendChild(document.createTextNode(" / "));
                        } else if (Self.additionalNode !== undefined) {
                            $(Self.nodeId).appendChild(Self.additionalNode);
                        }
                    }
                    Self._cancelDraw = false;
                }
            });
        });
    },
    
    drawSearch: function (module, searchTerm) {
        if (!this.nodeId || this.module !== module) {
            return;
        }
        this.clear();
        //#. Title for search results (displayed instead of a folder name).
        //#. %s is the search term.
        //#, c-format
        var myNode = newnode("span", 0, 0, [ addTranslated(format(_("Results for \"%s\""), searchTerm)) ]);
        $(this.nodeId).appendChild(myNode);
    },
    clear: function () {
        if (!$(this.nodeId)) {
            return;
        }
        var i = 0, $i = this.events.length;
        for (; i < $i; i++) {
            removeDOMEvent($(this.nodeId), this.events[i][0], this.events[i][1]);
        }
        this.events = [];
        removeChildNodes($(this.nodeId));
        if (this.modify_cb) {
            ox.api.folder.dispatcher.unbind("modify", this.modify_cb);
            this.modify_cb = null;
        }
    }
};

/**
 * Constructor of the subscribe E-Mail folder dialog
 */

window.SubscribeFolder = (function () {
    
    var Class = function () {
        
        var self = this;
        this.changedFolders = []; // temp storage for changed items
        
        // ox.api.cache.mailSubscription.clear();
        
        this.listFolder = function (list, node) {
            // clear content
            jQuery(node).empty();
            // loop over folders
            var i = 0, $l = list.length, folder;
            for (; i < $l; i++) {
                folder = list[i];
                if (ox.api.folder.is("mail", folder)) {
                    self.addFolderToList(folder, node);
                }
            }
            // stop being busy
            jQuery("#subscribe_folder_status").hide();
        };
        
        this.addFolderToList = function (folder, contentNode) {
            
            var sNode = newnode("td", { width: "18px" }, 0, [
                newnode("img", { verticalAlign: "middle" }, {
                    className: folder.subfolders ? "tree-children" : "tree-nochildren",
                    src: folder.subfolders ? getFullImgSrc("img/folderplus.png") : getFullImgSrc("img/noplus.gif")
                })
            ]);
            
            var iNode;
            
            var tNode = newnode("div", {
                marginLeft: "18px", 
                whiteSpace: "nowrap",
                display: "none"
            }, 0);

            sNode.onclick = function () {
                
                if (!folder.subfolders) {
                    return;
                }
                
                if (sNode.firstChild.src.match(/folderplus.png/)) {
                    
                    // be busy
                    jQuery("#subscribe_folder_status").css("opacity", "0.75").show();
                    
                    // get subfolder
                    ox.api.folder.getSubFolders({
                        folder: folder.id,
                        all: true,
                        tree: "0",
                        storage: ox.api.cache.subscribe,
                        success: function (data) {
                            sNode.parentNode.parentNode.parentNode.parentNode.appendChild(tNode);
                            sNode.firstChild.src = getFullImgSrc("img/folderminus.png");
                            sNode.firstChild.className = "tree-nochildren";
                            tNode.style.display = "block";
                            self.listFolder(data, tNode);
                        }
                    });
                } else {
                    sNode.parentNode.parentNode.parentNode.parentNode.removeChild(sNode.parentNode.parentNode.parentNode.nextSibling);
                    sNode.firstChild.src = getFullImgSrc("img/folderplus.png");
                    sNode.firstChild.className = "tree-children";
                }
            };
            
            function checkAllCheckboxes(id, bool_flag) {
                var all_cb = jQuery("#subscribe_folder_content INPUT[type='checkbox']");
                jQuery.each(all_cb, function (i, val) {
                    if (val.id.indexOf(id + "/") !== -1 && val.id !== id) {
                        if (!all_cb[i].disabled) {
                            jQuery(all_cb[i]).attr("checked", bool_flag);
                            self.addChangedItem({ id: val.id.replace(/^sub_folder_/,""),
                                data: { subscribed: bool_flag } });
                        }
                    }
                });
            }
            
            if (folder.own_rights & 0x3f80) {
                iNode = newnode("input", 0, {
                    type: "checkbox",
                    id: "sub_folder_" + folder.id, 
                    oxid: folder.id,
                    className: "noborder",
                    disabled: !(folder.own_rights & 0x3f80)
                });
                /*
                 * setting checked attribute only if folder is subscribed
                 */ 
                if (folder.subscribed === true) {
                    iNode.setAttribute("checked", "checked");
                }  
                iNode.onchange = function () {
                    self.addChangedItem({ id: iNode.oxid, data: { subscribed: iNode.checked } });
                    checkAllCheckboxes("sub_folder_" + iNode.oxid, iNode.checked);
                };
            }
            contentNode.appendChild(
                newnode("div", 0, { className: "tree-div" }, [
                    newnode("table", { borderCollapse: "collapse", borderWidth: "0", borderSpacing: "0", width: "100%" }, 0, [
                        newnode("tbody", { width: "100%" }, 0, [
                            newnode("tr", 0, 0, [
                                sNode,
                                newnode("td", { width: "18px" }, 0, [
                                    newnode("img", { verticalAlign: "middle" }, { src: getFullImgSrc("img/folder/folder_closed.gif") })
                                ]),
                                newnode("td", 0, 0, [ document.createTextNode(folder.title) ]),
                                newnode("td", { width: "30px", verticalAlign: "middle" }, 0, iNode ? [iNode] : 0)
                            ])
                        ])
                    ])
                ])
            );
            /*
             * Workaround to fix checkbox selection bug in IE
             * As long as the node hasn't been appended to the DOM tree IE6 will ignore
             * setting the checked attribute. A timeout takes care that the node will be
             * appended first.
             */
            if (folder.subscribed === true) {
                setTimeout(function () {
                    if (iNode) {
                        iNode.setAttribute("checked", "checked");
                    }
                }, 10);
            }
        };
        
        this.addChangedItem = function (item) {
            var i = 0;
            for (; i < self.changedFolders.length; i++) {
                if (self.changedFolders[i].id === item.id) {
                    self.changedFolders.splice(i, 1);
                    break;
                }
            }
            item.module = "folders";
            item.action = "update";
            item.tree = configGetKey("modules.folder.tree") || 0;
            self.changedFolders.push(item);
        };
        
        this.saveChanges = function () {
            
            if (self.changedFolders.length === 0) {
                self.closeWindow();
                return;
            }
            
            // be busy
            jQuery("#subscribe_folder_status").css("opacity", "0.75").show();
            
            // update
            ox.api.http.PUT({
                module: "multiple",
                data: this.changedFolders,
                appendColumns: false,
                success: function (response) {
                    // some errors?
                    var i = 0, $i = (response || []).length;
                    for (; i < $i; i++) {
                        if (response[i].error) {
                            ox.UINotifier.error(formatError(response[i]));
                        }
                    }
                    // close & update
                    self.closeWindow();
                    ox.api.cache.subscribe.clear();
                    ox.api.folder.refresh({ tree: "0" });
                    ox.api.folder.refresh({ tree: "1" });
                }
            });
        };
        
        this.openWindow = function (options) {
            // get mail accounts
            ox.api.account.all(function (accounts) {
                // init 
                var scrollBar = accounts.length > 3 ? 4 : null;
                var accSel = new ComboBox3(window, "subscribe_folder_account", "20em", 0, true, "cb_subscribe_folder_acc_chd", scrollBar);
                var i = 0, $l = accounts.length;
                for (; i < $l; i++) {
                    accSel.addElement(noI18n(accounts[i].name),
                        "default" + accounts[i].id);
                }
                accSel.getDomNode();
                // select any other default account
                if (options && options.selected) {
                    accSel.setKey("default" + options.selected);
                }
                
                register("cb_subscribe_folder_acc_chd", function () {
                    self.changeAccount(accSel.getKey());
                });
                self.changeAccount(accSel.getKey());
            });
        };
        
        this.changeAccount = function (id) {
            
            // clear changed folders
            this.changedFolders = [];
            
            // set modal
            ox.api.setModal(true);
            
            // be busy
            jQuery("#subscribe_folder_status").css("opacity", "0.75").show();
            
            // get sub folders
            ox.api.folder.getSubFolders({
                folder: id,
                all: true,
                tree: "0", // tree 0 only!
                storage: ox.api.cache.subscribe,
                success: function (list) {
                    // show node
                    showNode("subscribe_folder");
                    // list top-level folders
                    self.listFolder(list, $("subscribe_folder_content"));
                    centerPopupWindow($("subscribe_folder"));
                    $("subscribe_folder").style.display = "block";
                },
                error: function () {
                    // stop being busy
                    jQuery("#subscribe_folder_status").hide();
                }
            });
        };
        
        this.closeWindow = function () {
            unregister("cb_subscribe_folder_acc_chd", window.SubscribeFolder.changeAccount);
            this.changedFolders = [];
            ox.api.folder.refresh({ tree: "0" });
            ox.api.folder.refresh({ tree: "1" });
            hideNode("subscribe_folder");
            $("subscribe_folder").style.display = "none";
            ox.api.setModal(false);
        };
    };
    
    return new Class();
    
}());

register("OX_Folder_Cleared", function (folder_id) {
    var tmpFn = OXCache.join.add(),
        allCollections = OXMailMapping.getSubCollectionsfromObject({ folder_id : folder_id }),
        i = 0;
    for (; i < allCollections.length; i++) {
        OXMailMapping.deleteObjectsInternal(allCollections[i].objects);
    }
    tmpFn();
});

// mark all mails in folder as read
register("OX_Folder_Mark_Read", function (folder_id) {
    ox.api.http.PUT({
        module: "mail",
        params: {
            action: "update",
            folder: folder_id
        },
        data: { flags: 32, value: true },
        success: function (daten) {
            var lock = OXCache.join.add(),
                map = OXMailMapping,
                col = map.getSubCollectionsfromObject({ folder_id: folder_id }),
                i = 0,
                $i = col.length;
            // loop over object & update
            for (; i < $i; i++) {
                map.editObjectsInternal(col[i].objects, null, true, [map.flagmapping.seen]);
            }
            lock();
            // this works fine on current folder -
            // not if user just right clicks on folder to mark as read, so...
            ox.api.folder.update({
                folder: folder_id,
                local: true, // triggers "modify" event which triggers folder tree repaint
                data: { unread: 0 }
            });
        }
    });
});

fileloaded();
/**
 * 
 * All content on this website (including text, images, source
 * code and any other original works), unless otherwise noted,
 * is licensed under a Creative Commons License.
 * 
 * http://creativecommons.org/licenses/by-nc-sa/2.5/
 * 
 * Copyright (C) Open-Xchange Inc., 2006-2012
 * Mail: info@open-xchange.com 
 * 
 * @author Jan Finsel <jan.finsel@open-xchange.com>
 * 
 */

var globalalert = new AlertPopup();
var globalcreate= new CreatePopup();
function AlertPopup() {    
    this.header;
    this.buttons;
    this.text;
    this.div;
    this.cbresize;
    this.closecb;
    this.cbok;
    this.cbcancel;
    this.cbyes;
    this.cbno;
    this.cbsave;
    this.cbseries;
    this.cbappointment;
    this.cbnocomment;
    this.visibleButtons=[];
}
AlertPopup.SERIESBUTTON=50;
AlertPopup.OK=1;
AlertPopup.CANCEL=2;
AlertPopup.YES=4;
AlertPopup.NO=8;
AlertPopup.APPOINTMENT=16;
AlertPopup.SERIES=32;
AlertPopup.NOCOMMENT=64;
AlertPopup.EDIT=128;
AlertPopup.CONTINUE=256;
AlertPopup.VERSION=67;
AlertPopup.OKCANCEL=3;
AlertPopup.YESNO=12;
AlertPopup.CONTINUECANCEL=258;
AlertPopup.SAVE=512;

if (init && init.i18n) delete init.i18n["alert_window-h-c"];

AlertPopup.open = function () {
    globalalert.visibleButtons = [];
    /*
     * remove keydown event just in case!
     */
    removeDOMEvent(body, "keydown", AlertPopup.key_down);
    if ("Hover" in window) Hover.disable();
    ox.api.setModal(true);
    //showNode("modal-dialog");
    //$("modal-dialog").style.display="block";
    showNode("disable_confirm_window");
    $("disable_confirm_window").style.display="block";
    /*
     * enable / disable required buttons
     */
    if ((globalalert.buttons %  2) >= 1) { 
        $("alert_window_ok").style.display="block";
        setFocus($("alert_window_ok"));
        globalalert.visibleButtons.push({id: "alert_window_ok", evt: AlertPopup.commitok});
    } else { 
        $("alert_window_ok").style.display="none"; 
    }
    if ((globalalert.buttons %  4) >= 2) { 
        $("alert_window_cancel").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_cancel", evt: AlertPopup.commitcancel});
    } else { 
        $("alert_window_cancel").style.display="none"; 
    }
    if ((globalalert.buttons %  8) >= 4) { 
        $("alert_window_yes").style.display="block"; 
        setFocus($("alert_window_yes"));
        globalalert.visibleButtons.push({id: "alert_window_yes", evt: AlertPopup.commityes});
    } else { 
        $("alert_window_yes").style.display="none"; 
    }
    if ((globalalert.buttons % 16) >= 8) { 
        $("alert_window_no").style.display="block";
        globalalert.visibleButtons.push({id: "alert_window_no", evt: AlertPopup.commitno});
    } else { 
        $("alert_window_no").style.display="none"; 
    }    
    if ((globalalert.buttons % 64) >= 32) { 
        $("alert_window_series").style.display="block";
        globalalert.visibleButtons.push({id: "alert_window_series", evt: AlertPopup.commitseries});
    } else { 
        $("alert_window_series").style.display="none"; 
    }
    if ((globalalert.buttons % 32) >= 16) { 
        $("alert_window_appointment").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_appointment", evt: AlertPopup.commitappointment});
    } else { 
        $("alert_window_appointment").style.display="none"; 
    }
    if ((globalalert.buttons % 128) >= 64) { 
        $("alert_window_nocomment").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_nocomment", evt: AlertPopup.commitnocomment});
    } else { 
        $("alert_window_nocomment").style.display="none"; 
    }
    if ((globalalert.buttons % 256) >= 128) { 
        $("alert_window_edit").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_edit", evt: AlertPopup.commitedit});
    } else { 
        $("alert_window_edit").style.display="none"; 
    }
    if ((globalalert.buttons % 512) >= 256) { 
        $("alert_window_continue").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_continue", evt: AlertPopup.commitcontinue});
    } else { 
        $("alert_window_continue").style.display="none"; 
    }
    if ((globalalert.buttons % 1024) >= 512) { 
        $("alert_window_save").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_save", evt: AlertPopup.commitsave});
    } else { 
        $("alert_window_save").style.display="none"; 
    }
    if (globalalert.SHOWAGAIN) { 
        $("alert_window_showagain").style.display="block"; 
        globalalert.visibleButtons.push({id: "alert_window_showagain_value", evt: function() {}});
    } else { 
        $("alert_window_showagain").style.display="none"; 
    }
    showNode("alert_window");
    $("alert_window-h-c").firstChild.data = globalalert.header;
    removeChildNodes($("alert_window_text"));
    if (globalalert.div) {        
        $("alert_window_text").appendChild(globalalert.div);
    } else {
        var nText = globalalert.text.split("\n");
        for (var i in nText) {
            $("alert_window_text").appendChild(document.createTextNode(nText[i]));
            $("alert_window_text").appendChild(newnode("br"));
        }
    }
    centerPopupWindow($("alert_window"));    
    $("alert_window").style.display="block";
    addDOMEvent(body, "keydown", AlertPopup.key_down);
    // bind to resize
    if (!globalalert.cbresize) {
        globalalert.cbresize = function() {
            // bind to resize to prevent dialog moved out of view
            // bug #19950 - header offset needs to be checked as its top
            // position is -x :(
            var header = jQuery(".popup_border_lo", $("alert_window"));
            if (header && header.offset().top < 0) {
                jQuery($("alert_window")).css({ top: (header.height() + 1) + "px" });
            } 
        };
        jQuery(window).bind('resize', globalalert.cbresize);
    }
}
AlertPopup.close = function () {
    if ("Hover" in window) Hover.enable();
    removeDOMEvent(body, "keydown", AlertPopup.key_down);
    ox.api.setModal(false);
    hideNode("alert_window");
    $("alert_window").style.display="none";
    hideNode("disable_confirm_window");
    $("disable_confirm_window").style.display="none";
    var childs = $("modal-dialog").childNodes;
    var closeall=true;
    for(var i=0;i<childs.length;i++) {
        if(childs[i].tagName && childs[i].tagName == "DIV") {
            if(childs[i].style.display && childs[i].style.display != "none" ) {
                closeall=false;
            }
        }
    }
    globalalert.visibleButtons = {};
    if (closeall === false) {
        ox.api.setModal(true);
    }
    if (globalalert.closecb) {
        globalalert.closecb();
    }
    // unbind resize
    if (globalalert.cbresize) {
        jQuery(window).unbind('resize', globalalert.cbresize);
        globalalert.cbresize = undefined;
    }
}
AlertPopup.commityes = function () {
    AlertPopup.close();
    if(globalalert.cbyes) { globalalert.cbyes(); }
}
AlertPopup.commitno = function () {
    AlertPopup.close();
    if(globalalert.cbno) { globalalert.cbno(); }
}
AlertPopup.commitcancel = function () {
    AlertPopup.close();
    if(globalalert.cbcancel) { globalalert.cbcancel(); }
}
AlertPopup.commitok = function () {
    if (!globalalert.leaveOpen) {
        AlertPopup.close();
    }
    if(globalalert.cbok) { globalalert.cbok(); }
}
AlertPopup.commitseries = function () {
    AlertPopup.close();
    if(globalalert.cbseries) { globalalert.cbseries(); }
}
AlertPopup.commitnocomment = function () {
    AlertPopup.close();
    if(globalalert.cbnocomment) { globalalert.cbnocomment(); }
}
AlertPopup.commitappointment = function () {
    AlertPopup.close();
    if(globalalert.cbappointment) { globalalert.cbappointment(); }
}
AlertPopup.commitedit = function () {
    AlertPopup.close();
    if(globalalert.cbedit) { globalalert.cbedit(); }
}
AlertPopup.commitsave = function () {
    AlertPopup.close();
    if(globalalert.cbsave) { globalalert.cbsave(); }
}
AlertPopup.key_down = function(e) {
    switch (e.keyCode || e.which) {
        case 27: // esc
            AlertPopup.close();
            break;
        case 9: // tab
            if (!globalalert.visibleButtons.length) break;
            if (focusedElement != null) {               
                for (var i=0; i<globalalert.visibleButtons.length; i++) {
                    if (globalalert.visibleButtons[i].id == focusedElement.id) {
                        if (i+1 < globalalert.visibleButtons.length) {
                            setFocus($(globalalert.visibleButtons[i+1].id));
                        } else {
                            setFocus($(globalalert.visibleButtons[0].id));
                        }
                        break;
                    }
                }
            } else {
                setFocus($(globalalert.visibleButtons[0].id));
            }
            break;
        case 13: // enter 
            if (!globalalert.visibleButtons.length) break;
            if (focusedElement != null) {               
                for (var i=0; i<globalalert.visibleButtons.length; i++) {
                    if (globalalert.visibleButtons[i].id == focusedElement.id) {
                        globalalert.visibleButtons[i].evt();
                        break;
                    }
                }
            }
            break;         
        default:
            return;        
    }
    stopEvent(e);
    return false;
}
AlertPopup.prototype = 
{
    setOK : function (cb) {
        this.cbok=cb;
    },
    setCancel : function (cb) {
        this.cbcancel=cb;
    },
    setYes : function (cb) {
        this.cbyes=cb;
    },
    setNo : function (cb) {
        this.cbno=cb;
    },
    setSeries: function (cb) {
        this.cbseries=cb;
    },
    setAppointment: function (cb) {
        this.cbappointment=cb;
    },
    setClose: function (cb) {
        this.closecb=cb;
    },
    setNoComment : function(cb) {
        this.cbnocomment=cb;
    },
    setEdit: function(cb) {
        this.cbedit=cb;
    },
    setSave: function(cb) {
        this.cbsave=cb;
    }
};

function CreatePopup() {    
    this.header;
    this.buttons;
    this.text;
    this.inputtext;
    this.inputvalue;
    this.closecb;
    this.cbok;
    this.cbcancel;
    this.cbadvanced;
    this.cbnocomment;
}
CreatePopup.OK=1;
CreatePopup.CANCEL=2;
CreatePopup.NOCOMMENT=4;
CreatePopup.VERSION=7;
CreatePopup.ADVANCED=64;
CreatePopup.ALL=67;

CreatePopup.open = function () {
    ox.api.setModal(true);
    //showNode("modal-dialog");
    //$("modal-dialog").style.display="block";
    showNode("disable_confirm_window");
    $("disable_confirm_window").style.display="block";
    if ((globalcreate.buttons %  4) >= 2) { $("create_window_cancel").style.display="block"; } else { $("create_window_cancel").style.display="none"; }
    if ((globalcreate.buttons %  8) >= 4) { $("create_window_nocomment").style.display="block"; } else { $("create_window_nocomment").style.display="none"; }
    if ((globalcreate.buttons %  2) >= 1) { $("create_window_ok").style.display="block"; } else { $("create_window_ok").style.display="none"; }
    
    if ((globalcreate.buttons % 128) >= 64) { $("create_window_advanced").style.display="block"; } else { $("create_window_advanced").style.display="none"; }
    
    $("create_window-h-c").firstChild.data = globalcreate.header;
    $("create_window_text").firstChild.data = globalcreate.text;
    if(globalcreate.inputtype == "input") {
        $("create_window_table_textarea").style.display="none";
        $("create_window_table_text_input").style.display="block";
        if (globalcreate.inputtext) {
            $("create_window_text_input").style.display="";
            $("create_window_text_input").firstChild.data = globalcreate.inputtext;
        } else {
            $("create_window_text_input").style.display="none";
        }    
        $("create_window_text_field").value=globalcreate.inputvalue;
        setTimeout(function() { 
            if ($("create_window_text_field") && $("create_window_text_field").focus()) $("create_window_text_field").focus(); 
        },500);
    } else if(globalcreate.inputtype == "textarea") {
        $("create_window_table_textarea").style.display="block";
        $("create_window_table_text_input").style.display="none";
        $("create_window_textarea").firstChild.data = globalcreate.inputtext;    
        $("create_window_textarea_field").value=globalcreate.inputvalue;
        setTimeout(function() { 
            if ($("create_window_textarea_field") && $("create_window_textarea_field").focus()) $("create_window_textarea_field").focus(); 
        },500);
    }
    showNode("create_window");
    centerPopupWindow($("create_window"));
    $("create_window").style.display="block";
}
CreatePopup.close = function () {
    ox.api.setModal(false);
    hideNode("create_window");
    $("create_window").style.display="none";
    hideNode("disable_confirm_window");
    $("disable_confirm_window").style.display="none";
    var childs= $("modal-dialog").childNodes;
    var closeall=true;
    for(var i=0;i<childs.length;i++) {
        if(childs[i].tagName && childs[i].tagName == "DIV") {
            if(childs[i].style.display && childs[i].style.display != "none" ) {
                closeall=false;
            }
        }
    }
    if (closeall === false) {
        ox.api.setModal(true);
    }
    if(globalcreate.closecb) { globalcreate.closecb(); }

}
CreatePopup.commitcancel = function () {
    CreatePopup.close();
    if(globalcreate.cbcancel) { globalcreate.cbcancel(); }
}
CreatePopup.commitok = function () {
    CreatePopup.close();
    if(globalcreate.cbok) { globalcreate.cbok(); }
}
CreatePopup.commitnocomment = function () {
    CreatePopup.close();
    if(globalcreate.cbnocomment) { globalcreate.cbnocomment(); }
}
CreatePopup.commitadvanced = function () {
    CreatePopup.close();
    if(globalcreate.cbadvanced) { globalcreate.cbadvanced(); }
}

CreatePopup.prototype = 
{
    setOK : function (cb) {
        this.cbok=cb;
    },
    setCancel : function (cb) {
        this.cbcancel=cb;
    },
    setAdvanced: function (cb) {
        this.cbadvanced=cb;
    },
    setClose: function (cb) {
        this.closecb=cb;
    },
    setNoComment : function (cb) {
        this.cbnocomment=cb;
    }
}
function newAlert(headertitle,alerttext,okcb,div) {
    globalalert.header=headertitle;
    globalalert.buttons=CreatePopup.OK;
    globalalert.text=alerttext;
    globalalert.div=div;
    globalalert.closecb=null;
    globalalert.cbok=okcb;
    globalalert.cbcancel=null;
    globalalert.cbnocomment=null;
    globalalert.cbyes=null;
    globalalert.cbno=null;
    globalalert.cbsave=null;
    AlertPopup.open();
}
function newConfirm(headertitle,contenttext,buttons,cbok,cbcancel,cbyes,cbno,cbseries,cbappointment,cbclose,cbnocomment,cbedit,div) {
    globalalert.header=headertitle;
    globalalert.buttons=buttons;
    globalalert.text=contenttext;
    globalalert.div=div;
    globalalert.closecb=cbclose;
    globalalert.cbok=cbok;
    globalalert.cbappointment=cbappointment;
    globalalert.cbseries=cbseries;
    globalalert.cbcancel=cbcancel;
    globalalert.cbedit=cbedit;
    globalalert.cbyes=cbyes;
    globalalert.cbno=cbno;
    globalalert.cbnocomment=cbnocomment;
    AlertPopup.open();    
}
function newInput(headertitle,contenttext,inputtext,inputvalue,buttons,cbok,cbcancel,cbadvanced,cbclose,
                                                                                    inputType,cbnocomment) {
    globalcreate.header=headertitle;
    globalcreate.text=contenttext;
    globalcreate.inputtext=inputtext;
    if(inputvalue)
        globalcreate.inputvalue=inputvalue;
    else
        globalcreate.inputvalue="";
    globalcreate.buttons=buttons;
    globalcreate.closecb=cbclose;
    globalcreate.cbok=cbok;
    globalcreate.cbcancel=cbcancel;
    globalcreate.cbnocomment=cbnocomment;
    globalcreate.cbadvanced=cbadvanced;
    if(inputType)
        globalcreate.inputtype=inputType;
    else
        globalcreate.inputvalue="input";
    CreatePopup.open();    
}
function newAppointmentObject(appointmentobject,myok,mycancel,myadvanced,myclose) {
    if(appointmentobject.start_date && appointmentobject.end_date) {
        var mystring=formatDate(appointmentobject.start_date,"datetime")+ " - "+formatDate(appointmentobject.end_date,"datetime"); 
        newInput(_("New Appointment"),mystring, pgettext("description", "Title"),"",CreatePopup.ALL,myok,mycancel,myadvanced,myclose);
    }
}
register("Loaded",function() {
    addOnClose($('alert_window'),function () {
    	delete(globalalert.leaveOpen);
        AlertPopup.close();
    });
    addOnClose($('create_window'),function () { 
        CreatePopup.close(); 
    });
});
fileloaded();
